USE Televis

DECLARE @sqlCmd nvarchar (4000)

-- Creates a view for the data in the SequentialData table that is oldest than 1 WEEK.
IF NOT EXISTS(SELECT [name] FROM sys.views WHERE [name] = 'ViewOldSequentialData')          
BEGIN 	
    SELECT @sqlCmd = 'CREATE VIEW [dbo].[ViewOldSequentialData] AS (SELECT ID FROM [Televis].[dbo].[SequentialData] WITH (NOLOCK) WHERE Timestamp < DATEADD(WEEK, -1, GETDATE()))'
	-- Must use "execute SQL command " because otherwise a "IF NOT EXISTS(...) CREATE VIEW" fails as the CREATE VIEW must be the first instruction of a block.
    EXEC sp_executesql @sqlCmd

	IF NOT (SELECT DATABASE_PRINCIPAL_ID('SothisViewer')) IS NULL
		GRANT SELECT ON [dbo].[ViewOldSequentialData] TO SothisViewer
END

-- Removes oldest data from the SequentialData table.
DELETE FROM [dbo].[ViewOldSequentialData]

-- Creates a view for data in the HistoricalData table that is older than 1 YEAR.
IF NOT EXISTS(SELECT [name] FROM sys.views WHERE [name] = 'ViewOldHistoricalData')
BEGIN   
    -- The following expression (included in view), is the minimum date between "1 year ago" and "the first week still present in the DB".
    SELECT @sqlCmd = 'CREATE VIEW [dbo].[ViewOldHistoricalData] AS
      (SELECT ID FROM [Televis].[dbo].[HistoricalData] WITH (NOLOCK)
      WHERE Timestamp <
            (SELECT CASE WHEN DATEADD(DAY, -366, GETDATE()) > DATEADD(DAY, +7, (SELECT MIN([Timestamp]) FROM [dbo].[HistoricalData]))
                  THEN DATEADD(DAY, +7, (SELECT MIN([Timestamp]) FROM [dbo].[HistoricalData]))
                  ELSE DATEADD(DAY, -366, GETDATE()) END AS MinDate))
'
    -- Must use "execute SQL command " because otherwise a "IF NOT EXISTS(...) CREATE VIEW" fails as the CREATE VIEW must be the first instruction of a block.
    EXEC sp_executesql @sqlCmd

    IF NOT (SELECT DATABASE_PRINCIPAL_ID('SothisViewer')) IS NULL GRANT SELECT ON [dbo].[ViewOldHistoricalData] TO SothisViewer
END

-- Removes oldest data from the HistoricalData table.
DELETE FROM [dbo].[ViewOldHistoricalData]

FFFF1711