﻿window.addEvent('domready', function() {
    if ($('boxToolsWidgetLayout')) { this.layoutManager = new layoutManager(); }
});

function loadProperties() {
    this.layoutManager.loadProperties();
};

function refresh(from, to) {
    this.layoutManager.refresh(from, to);
};

function refreshDevice(key) {
    this.layoutManager.refreshDevice(key);
};

function sendCommand(key) {
    this.layoutManager.sendCommand(key);
};

function keyUpOn(e, elementKey) {
    var unicode = e.keyCode ? e.keyCode : e.charCode;
    if (unicode == 13) {
        this.layoutManager.writeParameter(elementKey);
    }
};

function rebuild() {
    new Request({ url: '/LayoutInvocation.rix', onComplete: window.location.reload() })
            .send("modality=ajax&req=rebuild&id=" + this.layoutManager.layoutId);
};

var layoutManager = new Class({
    initialize: function() {
        // Space minimize/maximize togglers.
        this.togspacemax = document.getElement('.wkaToolbarArea a.watRvdMaximize');
        if (this.togspacemax) {
            this.togspacemax.addEvent('click', this.spacetoggle.pass('max', this));
            this.togspacemax.onclick = function() { return false; };
        }
        this.togspacemin = document.getElement('.wkaToolbarArea a.watRvdMinimize');
        if (this.togspacemin) {
            this.togspacemin.addEvent('click', this.spacetoggle.pass('min', this));
            this.togspacemin.onclick = function() { return false; };
            this.togspacemin.setStyle('display', 'none');
        }

        // Client-side reload parameters.
        this.reloadParametersButton = document.getElement('.wkaToolbarArea a.watLayoutReloadParameters');
        if (this.reloadParametersButton) {
            this.reloadParametersButton.addEvent('click', this.pollParameters.bindWithEvent(this));
            this.reloadParametersButton.onclick = function() { return false; };
        }

        this.loading = $('labelLoading');
        // Loads interface (ajax)
        new Request({ url: '/LayoutInvocation.rix', onComplete: this.loadInterface.bindWithEvent(this) })
            .send("modality=ajax&req=init");

        var captions = $$('.caption');
        captions.each(function(c) {
            c.addEvents({
                mouseenter: function() {
                    this.addClass('unfolded');
                },
                mouseleave: function() {
                    this.removeClass('unfolded');
                }
            });
        });

        var myTips = new Tips($$('.tipped'), {
            text: "tip",
            title: "tipTitle",
            showDelay: 500
        });

        myTips.removeEvents('show').addEvent('show', function(tip, el) {
            tip.fade('in');
        });
        myTips.removeEvents('hide').addEvent('hide', function(tip, el) {
            tip.fade('hide');
        });
    },

    spacetoggle: function(mode) {
        switch (mode) {
            case 'max':
                document.getElement('#menuGeneral').setStyle('display', 'none');
                this.togspacemax.setStyle('display', 'none');
                this.togspacemin.setStyle('display', '');
                break;
            case 'min':
                document.getElement('#menuGeneral').setStyle('display', '');
                this.togspacemax.setStyle('display', '');
                this.togspacemin.setStyle('display', 'none');
                break;
        }
    },

    loadProperties: function() {
        this.layoutId = parseInt($('layoutId').value);
        this.chevron = parseInt($('layoutChevron').value);
        this.maxKey = parseInt($('layoutMaxKey').value);
        this.pollTime = parseInt($('layoutPollTime').value);
        this.acceleratedPollTime = 1000;
        this.waitingText = $('layoutWaitingText').value;
        this.timeoutText = $('layoutTimeoutText').value;
        if (this.chevron < 0) this.chevron = 0;
        if (this.chevron > this.maxKey) this.chevron = 0;
        this.current = 0;

        this.accelerated = false;
        this.pollStop();
        this.updatePollTimer();
    },

    setSpeed: function(accelerate) {
        if (accelerate == this.accelerated) return;
        this.accelerated = accelerate;
        this.loading.toggleClass('acc', accelerate);
        this.pollStop();
        this.updatePollTimer();
    },

    nowIsServing: function(who) {
        this.occupied = true;
        this.nowServing = who;
        this.loading.fade("show");
    },

    nowIsFree: function() {
        this.nowServing = "";
        this.loading.fade("out");
        this.occupied = false;
    },

    isOccupied: function() {
        return this.occupied;
    },

    loadInterface: function(request) {
        /* store the JSON data response */
        //this.value = JSON.decode(request);
        // TODO: infer from value.
        this.nowIsFree();

        this.loadProperties();

        // Stops polling on page unload.
        window.onbeforeunload = this.pollStop;
        window.onunload = this.pollStop;

        // Invokes initial parameter polling.
        this.pollParameters();
    },

    pollParameters: function() {
        if (this.isOccupied()) return;
        this.setSpeed(true);
        this.nowIsServing("many");

        new Request({ url: '/LayoutInvocation.rix', onComplete: this.updateChevron.bindWithEvent(this), timeout: 5000 })
            .send("modality=ajax&req=pr&id=" + this.layoutId);
    },

    updatePollTimer: function() {
        // start polling display update data
        if (this.accelerated) {
            this.pollTimer = this.pollStatus.periodical(this.acceleratedPollTime, this);
        } else {
            this.pollTimer = this.pollStatus.periodical(this.pollTime, this);
        }
    },

    refresh: function(from, to) {
        if (this.isOccupied()) return;
        this.nowIsServing("many");

        var requiredKeys = from <= to ? from + "-" + to : to + "-" + from;

        // load mask data (ajax)
        new Request({ url: '/LayoutInvocation.rix', onComplete: this.updateChevron.bindWithEvent(this), timeout: 5000 })
            .send("modality=ajax&req=chevron&id=" + this.layoutId + "&keys=" + requiredKeys);
    },

    pollStatus: function() {
        if (this.isOccupied()) return;
        this.nowIsServing("many");

        //Evaluates the keys to request.
        var from = this.chevron == 0 ? 0 : this.current;
        var to = this.chevron == 0 ? this.maxKey : this.current + this.chevron;
        if (to > this.maxKey) to = (to - 1) % (this.maxKey + 1);
        this.current = (to + 1);
        if (this.current > this.maxKey) this.current = 0;

        var requiredKeys = from + "-" + to;

        // load mask data (ajax)
        new Request({ url: '/LayoutInvocation.rix', onComplete: this.updateChevron.bindWithEvent(this), timeout: 5000 })
            .send("modality=ajax&req=chevron&id=" + this.layoutId + "&keys=" + requiredKeys);
    },

    pollStop: function(mode) {
        if (this.pollTimer) { this.pollTimer = $clear(this.pollTimer); }
    },

    updateChevron: function(response) {
        resourceValues = JSON.decode(response);
        this.updateResourceValues(resourceValues);
        this.nowIsFree();
        if (resourceValues) this.setSpeed(resourceValues.AcceleratePolling);
    },

    updateResourceValues: function(resourceValues) {
        if (resourceValues == null) return;

        for (key in resourceValues.Text) {
            var element = $('lot' + key);
            if (element) {
                var text = "";
                if (resourceValues.Status[key] != null) {
                    text = resourceValues.Status[key];
                } else {
                    text = resourceValues.Text[key];
                }
                if (element.match('input') && document.activeElement != element) {
                    element.set('value', text);
                }
                if (element.match('label')) element.innerHTML = text;
            }
        }

        for (key in resourceValues.Image) {
            var name = 'loi' + key;
            if ($(name)) {
                var src = resourceValues.Image[key];
                $(name).set('src', src);
            }
        }

        for (key in resourceValues.Numeric) {
            var name = 'lon' + key;
            if ($(name)) {
                var percent = resourceValues.Numeric[key];
                var property = '';
                if ($(name).hasClass('vertical')) { property = 'height'; percent = 100 - percent; }
                if ($(name).hasClass('horizontal')) { property = 'width'; }
                $(name).setStyle(property, percent + "%");
            }
        }
    },

    beginFeedbackWaiting: function(key) {
        var feedback = $(key);
        if (feedback) {
            var fx = feedback.get('tween');
            fx.cancel();
            feedback.innerHTML = this.waitingText;
            feedback.removeClass('ok');
            feedback.removeClass('error');
            feedback.removeClass('waiting');
            feedback.addClass('waiting');
            feedback.fade('show');
        }
    },

    endFeedbackWaiting: function(key, text, cssClass) {
        var feedback = $(key);
        if (feedback) {
            feedback.innerHTML = text;
            feedback.removeClass('ok');
            feedback.removeClass('error');
            feedback.removeClass('waiting');
            feedback.addClass(cssClass);
            feedback.set('tween', { duration: '5000' });
            feedback.fade('out');
        }
    },

    sendCommand: function(key) {
        if (this.isOccupied()) return;
        this.nowIsServing(key);

        $('loc' + key).set('disabled', true);
        this.beginFeedbackWaiting('lofb' + key);

        var ajax = new Request({
            url: '/LayoutInvocation.rix',
            timeout: 10000,
            onTimeout: function() { this.updateCommand(null); } .bind(this),
            onComplete: function(response) { this.updateCommand(response); } .bind(this)
        });

        ajax.send("modality=ajax&req=cmd&id=" + this.layoutId + "&key=" + key);
    },

    writeParameter: function(key) {
        if (this.isOccupied()) return;
        this.nowIsServing(key);

        var element = $('lot' + key);
        if (!element) return;

        element.set('disabled', true);
        var value = element.get('value');
        this.beginFeedbackWaiting('lofb' + key);

        var ajax = new Request({
            url: '/LayoutInvocation.rix',
            timeout: 10000,
            onTimeout: function() { this.updateParameter(null); } .bind(this),
            onComplete: function(response) { this.updateParameter(response); } .bind(this)
        });

        ajax.send("modality=ajax&req=pw&id=" + this.layoutId + "&key=" + key + "&value=" + encodeURIComponent(value));
    },

    updateCommand: function(response) {
        if ($('loc' + this.nowServing)) { $('loc' + this.nowServing).set('disabled', false); }

        if (response == null) {
            this.endFeedbackWaiting('lofb' + this.nowServing, this.timeoutText, 'error');
        } else {
            var commandResult = JSON.decode(response);
            if (commandResult != null) {
                this.updateResourceValues(commandResult.ResourceValues);
                this.endFeedbackWaiting('lofb' + this.nowServing, commandResult.Result, commandResult.CssClass);
            }
        }

        this.nowIsFree();
    },

    updateParameter: function(response) {
        if ($('lot' + this.nowServing)) { $('lot' + this.nowServing).set('disabled', false); }

        if (response == null) {
            this.endFeedbackWaiting('lofb' + this.nowServing, this.timeoutText, 'error');
        } else {
            var writeResult = JSON.decode(response);
            if (writeResult != null) {
                if (writeResult.Value != null) $('lot' + this.nowServing).set('value', writeResult.Value);
                this.updateResourceValues(writeResult.ResourceValues);
                this.endFeedbackWaiting('lofb' + this.nowServing, writeResult.Result, writeResult.CssClass);
            }
        }

        this.nowIsFree();
    },

    refreshDevice: function(key) {
        if (this.isOccupied()) return;
        this.nowIsServing(key);

        // load mask data (ajax)
        new Request({ url: '/LayoutInvocation.rix', onComplete: this.updateChevron.bindWithEvent(this), timeout: 5000 })
            .send("modality=ajax&req=dev&id=" + this.layoutId + "&key=" + key);
    }
});