function createTable(panelContainer, headers) {
    var table = new Element('table', { 'class': 'simpleTable' });
    table.set('cellpadding', '0');
    table.set('cellspacing', '0');
    table.inject(panelContainer);
    var thead = new Element('thead', {});
    thead.inject(table);
    var trH = new Element('tr', {});
    trH.inject(thead);
    for (var i = 0; i < headers.length; i++) {
        var th = new Element('th', { html: headers[i] });
        th.inject(trH);
    }
    var tbody = new Element('tbody', {});
    tbody.inject(table);
    return tbody;
}
var manageArgs = true;
var myClientDataEntry;
var argsTextWidth = "80px";
var ARGS_SEPARATOR = '|';
var COMMAND_SEPARATOR = '#';
var GROUP_SEPARATOR = '@';
var ITEM_SEPARATOR = '$';

var commandArgumentsManager = {
    initialize: function () {
        $('decCommand').addEvent('change', commandArgumentsManager.buildArgs);
    },

    setArguments: function (prefix, values) {
        for (i = 0; i < values.length; i++) {
            var currentControl = $(prefix + i);
            if (currentControl != null) {
                $(currentControl).value = values[i];
            }
        }
    },

    buildArgs: function (ddlName, target, prefix) {
        if (manageArgs) {
            var ddlName = "decCommand";
            var target = "gArgs";
            var prefix = "decArg";
            var ddlCommands = $(ddlName);
            if (ddlCommands.selectedIndex != -1) {
                var selectedCommandID = ddlCommands.options[ddlCommands.selectedIndex].value;
                var postBody = "modality=ajax" + "&exec=getArgumentsHTML" + "&commandID=" + selectedCommandID;
                new Request({
                    url: '/SchedulerActionUtilityInvocation.rix',
                    onComplete: function (request) {
                        commandArgumentsManager.insertArguments(request, target, prefix);
                    },
                    timeout: 5000
                }).send(postBody);
            }
        }
    },

    insertArguments: function (argumentList, target, prefix) {
        if (manageArgs) {
            var table = $(target);
            var argsArray = (argumentList == "" || argumentList == null) ? new String() : argumentList.split(ITEM_SEPARATOR);
            var currentRow = 0;
            argsControls = new Array();

            for (var i = table.rows.length; i > 0; i--) {
                table.deleteRow(i - 1);
            }

            for (i = 0; i < argsArray.length; i++) {
                var newRow = table.insertRow(-1);
                newRow.className = 'inputtext';
                var headerCell = document.createElement('th');
                headerCell.innerHTML = "<label>" + argsArray[i] + "</label>";
                newRow.appendChild(headerCell, textCell);
                var textCell = document.createElement('td');
                textCell.innerHTML = "<input style=\"width:" + argsTextWidth + "\" type=\"text\" value=\"\" id=\"" + prefix + i + "\" name=\"" + prefix + i + "\" />";
                newRow.appendChild(textCell);
            }
        }
    },

    loadDataEdit: function () {
        var selectedType = $('cActionType').selectedIndex;
        if (selectedType == 0) {
            var actionID = myClientDataEntry.getRecordID();
            var postBody = "modality=ajax" + "&exec=loadData" + "&actionID=" + actionID;
            new Request({
                url: '/SchedulerActionUtilityInvocation.rix',
                onComplete: function (request) {
                    var ddlCommand = $('decCommand');
                    ddlCommand.removeEvent('change', commandArgumentsManager.buildArgs);
                    var group = request.split(GROUP_SEPARATOR);
                    if (group.length >= 3) {
                        var args = group[2].split(ARGS_SEPARATOR);
                        selectValue(ddlCommand, group[0]);
                        if (manageArgs) {
                            commandArgumentsManager.insertArguments(group[1], 'gArgs', 'decArg');
                            commandArgumentsManager.setArguments('decArg', args);
                        }
                        ddlCommand.addEvent('change', commandArgumentsManager.buildArgs);
                    }
                },
                timeout: 5000
            }).send(postBody);
        }
    }
};


var dataExportManager = {
    initialize: function () {

    },
    loadDataEdit: function () {
        var selectedType = $('cActionType').selectedIndex;
        if (selectedType == 2) {
            var actionID = myClientDataEntry.getRecordID();
            var postBody = "modality=ajax" + "&exec=loadData" + "&actionID=" + actionID;
            new Request({
                url: '/SchedulerActionUtilityInvocation.rix',
                onComplete: function (request) {
                    var ddlCommand = $('decCommand');
                    ddlCommand.removeEvent('change', commandArgumentsManager.buildArgs);
                    var group = request.split(GROUP_SEPARATOR);
                    var args = group[2].split(ARGS_SEPARATOR);
                    selectValue(ddlCommand, group[0]);
                    if (manageArgs) {
                        commandArgumentsManager.insertArguments(group[1], 'gArgs', 'decArg');
                        commandArgumentsManager.setArguments('decArg', args);
                    }
                    ddlCommand.addEvent('change', commandArgumentsManager.buildArgs);
                },
                timeout: 5000
            }).send(postBody);
        }
    }
};

var parametersManager = {
    initialize: function () {
        this.messagePanel = $('parameters_message_panel').hide();
        this.listPanel = $('parameters_list_panel').hide();
        this.addPanel = $('parameters_add_panel').hide();

        this.inputControl = $('cParam');
        this.addBtn = $('parameters_add_btn');
        this.addInputLabel = $('parameters_add_input_label');
        this.addInputValue = $('parameters_add_input_value');
        var self = this;

        this.addBtn.addEvent('click', parametersManager.addData);
        $('warningParametersText').set('html', actionScheduler_uiDictionary["ParametersWarning"]);
    },

    displayData: function () {
        parametersManager.addPanel.hide();
        if (parametersManager.paramArray.length == 0) {
            parametersManager.messagePanel.show();
            parametersManager.listPanel.hide();
            return;
        }
        parametersManager.messagePanel.hide();
        parametersManager.listPanel.show();
        parametersManager.listPanel.set('html', '');

        var tbody = createTable(parametersManager.listPanel, [actionScheduler_uiDictionary["Label"], actionScheduler_uiDictionary["Value"]]);
        for (var index = 0; index < parametersManager.paramArray.length; index++) {
            var styleClass = '';
            if (index % 2 == 0)
                styleClass = 'odd';
            var tr = new Element('tr', { 'class': styleClass });
            var item = parametersManager.paramArray[index];

            var tdEntryLabel = new Element('td', { html: item.Label });
            tdEntryLabel.inject(tr);

            var tdEntryValue = new Element('td', { html: item.Value });
            tdEntryValue.inject(tr);

            tr.inject(tbody);
        }
    },

    editData: function () {

        parametersManager.addPanel.show();
        if (parametersManager.paramArray.length == 0) {
            parametersManager.messagePanel.show();
            parametersManager.listPanel.hide();
            return;
        }

        parametersManager.messagePanel.hide();
        parametersManager.listPanel.show();
        parametersManager.listPanel.set('html', '');

        var tbody = createTable(parametersManager.listPanel, [actionScheduler_uiDictionary["Label"], actionScheduler_uiDictionary["Value"], ""]);

        for (var index = 0; index < parametersManager.paramArray.length; index++) {
            var styleClass = '';
            if (index % 2 == 0)
                styleClass = 'odd';
            var tr = new Element('tr', { 'class': styleClass });
            var item = parametersManager.paramArray[index];

            var tdEntryLabel = new Element('td', { html: item.Label });
            tdEntryLabel.inject(tr);

            var tdEntryValue = new Element('td', { html: item.Value });
            tdEntryValue.inject(tr);

            var tdAction = new Element('td', {});
            var img = new Element('img', {
                src: '../media/img/Others/ActionSheduler/remove_btn.gif',
                width: '20px',
                height: '20px'
            }
                    );

            var btn = new Element('a', {
                href: '#',
                'class': 'removeBtn'
            });
            btn.set('data-id', index);
            img.inject(btn);

            btn.addEvent('click', function (event) {
                var id = $(this).get('data-id');
                parametersManager.deleteItem(id);
                event.preventDefault();
            });

            btn.inject(tdAction);
            tdAction.inject(tr);
            tr.inject(tbody);
        }
    },

    resetDataEdit: function () {
        parametersManager.paramArray = new Array();
        //parametersManager.paramArray.push({ Label: '???', Value: '???' });
        parametersManager.editData();
    },

    loadDataEdit: function () {
        var params = JSON.decode(parametersManager.inputControl.get('value'));
        if (params == null || params.length <= 0) {
            parametersManager.resetDataEdit();
            return;
        }
        parametersManager.paramArray = params;
        parametersManager.editData();
    },

    loadDataDisplay: function () {
        var params = JSON.decode(parametersManager.inputControl.get('value'));
        if (params == null || params.length <= 0) {
            parametersManager.paramArray = new Array();
            parametersManager.displayData();
            return;
        }
        parametersManager.paramArray = params;
        parametersManager.displayData();
    },

    addData: function (event) {
        var label = parametersManager.addInputLabel.get('value');
        var value = parametersManager.addInputValue.get('value');
        event.preventDefault();
        if (label == "" || value == "") {
            return;
        }

        var itemSubstituted = false;
        for (var i = 0; i < parametersManager.paramArray.length; i++) {
            if (parametersManager.paramArray[i].Label == '???' || parametersManager.paramArray[i].Label == label) {
                parametersManager.paramArray[i].Label = label;
                parametersManager.paramArray[i].Value = value;
                itemSubstituted = true;
            }
        }
        if (!itemSubstituted) {
            var item = { Label: label, Value: value };
            parametersManager.paramArray.push(item);
        }

        parametersManager.editData();
    },

    saveData: function () {
        return JSON.encode(parametersManager.paramArray);
    },

    deleteItem: function (index) {
        var newArray = new Array();
        for (var i = 0; i < parametersManager.paramArray.length; i++) {
            if (i != index)
                newArray.push(parametersManager.paramArray[i]);
        }
        parametersManager.paramArray = newArray;
        //        if (parametersManager.paramArray.length <= 0)
        //            parametersManager.paramArray.push({ Label: '???', Value: '???' });
        parametersManager.editData();
    }
};

var ValidationControl = new Class({
    initialize: function (controlName, regString, isRequired, condition) {
        this.control = $(controlName);
        this.regString = regString;
        this.isRequired = isRequired;
        this.condition = condition;
        this.validator = new Element('span', {
            html: '*',
            'class': 'validatorSymbol'
        });
        this.validator.inject(this.control, 'after');
        this.validator.hide();
    },
    validate: function () {
        if (this.isRequired) {
            if (this.control.get('value') == "") {
                this.validator.show();
                return false;
            } else {
                this.validator.hide();
                return true;
            }
        }
        if (this.regString != "") {
            var re = new RegExp(this.regString);
            if (!control.match(re)) {
                this.validator.show();
                return false;
            } else {
                this.validator.hide();
                return true;
            }
        }
        if (this.condition != null) {
            var ok = this.condition();
            this.validator.setStyle('display', ok ? 'none' : '');
            return ok;
        }
    },

    reset: function () {
        this.validator.hide();
    }
});

var baseValidationManager = {
    initialize: function () {
        this.validationControls = new Array();
    },

    addControl: function (controlID, regStr, isRequired) {
        var vc = new ValidationControl(controlID, regStr, isRequired);
        baseValidationManager.validationControls.push(vc);
    },
    validate: function (controlID) {
        var isValid = true;
        for (var index = 0; index < baseValidationManager.validationControls.length; index++) {
            if (!baseValidationManager.validationControls[index].validate()) {
                isValid = false;
            }
        }
        return isValid;
    },
    reset: function () {
        for (var index = 0; index < baseValidationManager.validationControls.length; index++) {
            baseValidationManager.validationControls[index].reset();
        }
    }
};

var timesManager = {
    initialize: function () {
        var self = this;

        this.messagePanel = $('times_message_panel').hide();
        this.listPanel = $('times_list_panel'); // .hide(); no! times_list_panel never hides
        // this.addPanel = $('times_add_panel').hide();

        // wire click after the editData ha built to table
        
        this.inputControl = $('cTimes');
        this.timeArray = new Array(); 
         
    },

    saveData: function () {
        var serializedData = "";
        for (var i = 0; i < timesManager.timeArray.length; i++) {
            serializedData += timesManager.timeArray[i] + ' ';
        }
        var stringLength = serializedData.length;
        if (stringLength > 0)
            serializedData = serializedData.substring(0, stringLength - 1);
        return serializedData;
    },

    deleteTime: function (index) {
        var time = timesManager.timeArray[index];
        timesManager.timeArray.erase(time);
        //        if (timesManager.timeArray.length <= 0)
        //            timesManager.timeArray.push('01:00');
        timesManager.editData();
    },

    addElement: function (time) {
        if (!timesManager.timeArray.contains(time) && time != "") {
            timesManager.timeArray.push(time);
        }
        timesManager.timeArray.sort();
        timesManager.editData();
    },

    addData: function (event) {
        var timeAddInput = $('times_add_input');
        var currentTime = timeAddInput.get('value');
        timesManager.addElement(currentTime);
        if (event)
            event.preventDefault();

    },

    editData: function () {

        // timesManager.addPanel.show();


        timesManager.messagePanel.hide();
        if (timesManager.timeArray.length == 0) {
            timesManager.messagePanel.show();
            // timesManager.listPanel.hide();
            // return;
        }

        // timesManager.listPanel.show();
        timesManager.listPanel.set('html', '');

        var tbody = createTable(timesManager.listPanel, [actionScheduler_uiDictionary["Time"], ""]);

        for (var index = 0; index < timesManager.timeArray.length; index++) {
            var styleClass = '';
            if (index % 2 == 0)
                styleClass = 'odd';
            var tr = new Element('tr', { 'class': styleClass });
            var currentTime = timesManager.timeArray[index];
            var tdEntry = new Element('td', { html: currentTime });
            tdEntry.inject(tr);
            var tdAction = new Element('td', {});
            var img = new Element('img', {
                src: '../media/img/Others/ActionSheduler/remove_btn.gif',
                width: '20px',
                height: '20px'
            }
                    );

            var btn = new Element('a', {
                href: '#',
                'class': 'removeBtn'
            });
            btn.set('data-id', index);
            img.inject(btn);
            btn.addEvent('click', function (event) {
                var id = $(this).get('data-id');
                timesManager.deleteTime(id);
                if (event)
                    event.preventDefault();
            });

            btn.inject(tdAction);
            tdAction.inject(tr);
            tr.inject(tbody);
        }

        // quite inefficient to re-create picker each time but easyest to maintain in code!
        var trAddTime = new Element('tr', { 'class': '' });
        var tdAddBtn = new Element('td', { html: '<input type="text" style="float: left; width: 40px;" name="times_add_input" id="times_add_input" />' });
        var tdSecondColumn = new Element('td', { html: '' });
        tdAddBtn.inject(trAddTime);
        tdSecondColumn.inject(trAddTime);
        trAddTime.inject(tbody);

        var timeAddInput = $('times_add_input');

        var newTimePicker = new DatePicker('times_add_input', {
            pickerClass: 'datepicker_vista',
            timePicker: true,
            timePickerOnly: true,
            inputOutputFormat: 'H:i',
            format: 'H:i',
            allowEmpty: true,
            onSelect: function () {
                timesManager.addData();
            }
        });

    },

    displayData: function () {

        // timesManager.addPanel.hide();

        if (timesManager.timeArray.length == 0) {
            timesManager.messagePanel.show();
            // timesManager.listPanel.hide();
            return;
        }
        timesManager.messagePanel.hide();
        // timesManager.listPanel.show();
        timesManager.listPanel.set('html', '');

        var tbody = createTable(timesManager.listPanel, [actionScheduler_uiDictionary["Time"]]);
        for (var index = 0; index < timesManager.timeArray.length; index++) {
            var styleClass = '';
            if (index % 2 == 0)
                styleClass = 'odd';
            var tr = new Element('tr', { 'class': styleClass });
            var currentTime = timesManager.timeArray[index];
            var tdEntry = new Element('td', { html: currentTime });
            tdEntry.inject(tr);
            tr.inject(tbody);
        } 

    },
    resetDataEdit: function () {
        timesManager.timeArray = new Array();
        //timesManager.timeArray.push('01:00');
        timesManager.editData();
    },
    loadDataEdit: function () {
        var params = timesManager.inputControl.get('value');
        if (params == null || params.length <= 0) {
            timesManager.resetDataEdit();
            return;
        }
        timesManager.timeArray = timesManager.inputControl.get('value').split(" ");
        timesManager.editData();
    },

    loadDataDisplay: function () {
        var currentText = timesManager.inputControl.get('value');
        if (currentText == null || currentText.length <= 0) {
            timesManager.timeArray = new Array();
            timesManager.displayData();
            return;
        }
        timesManager.timeArray = timesManager.inputControl.get('value').split(" ");
        timesManager.displayData();
    }
};

var startDatePicker;
var endDatePicker;
var retryTimeoutPicker;
var retryDurationPicker;
var periodPicker;
var myClientDataEntry;
var vcPeriodTime;
var vcPeriodInt;
var vcTimesList;
var vcCommand;
var vcWritingList;
var vcMapFileName;
var vcExportEmail;
var vcName;

function openPreview() {
    var scheduledActionId = myClientDataEntry.getRecordID();
    window.open('/Settings/FilterPreview.aspx' + '?scheduledActionID=' + scheduledActionId, 'Preview');
}

function update_export_period_visibility() {

    if ($('cActionType').selectedIndex != 2) //Lem 2 is Data export
    {
        return;
    }

    var selectedPeriod = $('cExportPeriod').selectedIndex;
    var count = $('cExportUndersampling').length;
    // Hides all except the first (no undersampling).
    for (var i = 1; i < count; i++) {
        $$('#cExportUndersampling option[value="' + i + '"]')[0].hide();
    }

    var query = "";
    var undersamplingLable = $('cExportUndersampling').getParent().getPrevious().getChildren()[0];

    window.onDemandSchedulationOption.hide();

    if (selectedPeriod == 0) {
        // { flash }
        $('cExportUndersampling').hide();
        undersamplingLable.hide();
        selectValue($('cExportUndersampling'), '0');
        // Sets daily schedulation
        window.periodicSchedulationOption.hide();
        window.dailySchedulationOption.show();
        window.weeklySchedulationOption.hide();
        selectValue($('cScheduleType'), '2');

    } else if (selectedPeriod == 1) {
        // { daily }
        $('cExportUndersampling').show();
        undersamplingLable.show();
        query = "modality=ajax&exec=getDailyUndersampling";
        // Sets daily schedulation
        window.periodicSchedulationOption.hide();
        window.dailySchedulationOption.show();
        window.weeklySchedulationOption.hide();
        selectValue($('cScheduleType'), '2');

    } else if (selectedPeriod == 2) {
        // { weekly }
        $('cExportUndersampling').show();
        undersamplingLable.show();
        query = "modality=ajax&exec=getWeeklyUndersampling";
        // Sets weekly schedulation
        window.periodicSchedulationOption.hide();
        window.dailySchedulationOption.hide();
        window.weeklySchedulationOption.show();
        selectValue($('cScheduleType'), '3');
    }

    update_export_variations_visibility();
    update_schedule_visibility();

    if (query == "") return;

    new Request(
        {
            url: '/SchedulerActionUtilityInvocation.rix',
            onComplete: function (request) {
                var list = JSON.decode(request);
                if (list == null) return;
                var currentSampling = $('cExportUndersampling').selectedIndex;
                var exists = false;
                for (var i = 0; i < list.length; i++) {
                    $$('#cExportUndersampling option[value="' + list[i] + '"]')[0].show();
                    if (currentSampling == list[i]) exists = true;
                }
                if (!exists) {
                    selectValue($('cExportUndersampling'), '0');
                }
            },
            timeout: 5000
        }).send(query);
}

function update_action_visibility() {
    var selectedType = $('cActionType').selectedIndex;
    var selectedScheudle = $('cScheduleType').selectedIndex;

    if (selectedType == 0) {
        // Command
        $('gCommand').show();
        $('gArgs').show();
        $('gParamMain').hide();
        $('gParamFileName').hide();
        $('gParameters').hide();
        $('gDataExportSettings').hide();
        $('gDataExportFolder').hide();
        $('gDataExportEmailRecipients').hide();
        $('gProfile').hide();
        $('gFilter').show();
        $('warningParameters').hide();

        window.periodicSchedulationOption.show();
        window.dailySchedulationOption.show();
        window.weeklySchedulationOption.show();
        window.onDemandSchedulationOption.hide();
        if (selectedScheudle == 3) {
            // On-demand is not a viable choice
            $('cScheduleType').selectedIndex = 0;
        }
        update_schedule_visibility();

    } else if (selectedType == 1) {
        // Parameter Writing
        $('gCommand').hide();
        $('gArgs').hide();
        $('gParamMain').show();
        $('gDataExportSettings').hide();
        $('gDataExportFolder').hide();
        $('gDataExportEmailRecipients').hide();
        $('gProfile').hide();
        $('gFilter').show();
        $('warningParameters').show();
        update_parmaWriter_visibility();

        window.periodicSchedulationOption.show();
        window.dailySchedulationOption.show();
        window.weeklySchedulationOption.show();
        window.onDemandSchedulationOption.hide();
        update_schedule_visibility();

    } else if (selectedType == 2) {
        // Data export        
        $('gCommand').hide();
        $('gArgs').hide();
        $('gParamMain').hide();
        $('gParamFileName').hide();
        $('gParameters').hide();
        $('gDataExportSettings').show();
        $('gDataExportFolder').show();
        $('gDataExportEmailRecipients').show();
        $('gProfile').show();
        $('gFilter').hide();
        $('cPeriodTime').set('value', '1.00:00:00');
        $('warningParameters').hide();
        periodPicker.refreshValue($('cPeriodTime').get('value'));
        checkExportEnabling();
        update_export_period_visibility();
    }
}

function update_parmaWriter_visibility() {
    var selectedType = $('cParamType').selectedIndex;
    var selectedTypeAction = $('cActionType').selectedIndex;
    if (selectedTypeAction != 1) {
        $('gParamFileName').hide();
        $('gParameters').hide();
        $('warningParameters').hide();
        return;
    }
    if (selectedType == 0) {// File Name
        $('gParamFileName').show();
        $('gParameters').hide();
        $('warningParameters').show();
    }
    else if (selectedType == 1) {//Parameters
        $('gParamFileName').hide();
        $('gParameters').show();
        $('warningParameters').show();
    }
}

function update_schedule_visibility() {
    var selectedType = $('cScheduleType').selectedIndex;

    if (selectedType == 0) {//Periodic
        $('GenericContainerStartDate').show();
        $('GenericContainerRetryDuration').show();
        $('GenericContainerRetryTimeout').show();
        $('gcPeriodTime').show();
        $('gcPeriodInt').hide();
        $('panel_times').hide();
        $('panel_days').hide();
    }
    else if (selectedType == 1) {// Daily
        $('GenericContainerStartDate').show();
        $('GenericContainerRetryDuration').show();
        $('GenericContainerRetryTimeout').show();
        $('gcPeriodTime').hide();
        $('gcPeriodInt').show();
        $('panel_times').show();
        $('panel_days').hide();
    }
    else if (selectedType == 2) { // Weekly
        $('GenericContainerStartDate').show();
        $('GenericContainerRetryDuration').show();
        $('GenericContainerRetryTimeout').show();
        $('gcPeriodTime').hide();
        $('gcPeriodInt').show();
        $('panel_times').show();
        $('panel_days').show();
    }
    else {// On demand
        $('GenericContainerStartDate').hide();
        $('GenericContainerRetryDuration').hide();
        $('GenericContainerRetryTimeout').hide();
        $('gcPeriodTime').hide();
        $('gcPeriodInt').hide();
        $('panel_times').hide();
        $('panel_days').hide();
    }
}

function update_export_variations_visibility() {
    var undersampling = $('cExportUndersampling').selectedIndex;
    var periodicity = $('cExportPeriod').selectedIndex;

    if (undersampling == 0 && periodicity != 0) {
        // { no undersampling and not a flash export }
        $('cExportVariations').getParent().show();
    } else {
        $('cExportVariations').getParent().hide();
    }
}

function setVisibility(array, value) {
    for (var i = 0; i < array.length; i++) {
        $(array[i]).style.display = value;
    }
}

function selectValue(control, value) {
    for (var i = 0; i < control.length; i++) {
        if (control.options[i].value == value) {
            control.selectedIndex = i;
        }
    }
}

function details_onSelect(event) {

    myClientDataEntry.exeRecordSelect(event);

    // same code #details_onSelect begin
    // Preview
    var previewBtn = document.getElement('div.wkaToolbarArea td.dosome a.watGenericPreview');
    if (previewBtn)
        previewBtn.removeClass('disabled');

    // Update Picker values
    retryTimeoutPicker.refreshValue($('cRetryTimeout').get('value'));
    retryDurationPicker.refreshValue($('cRetryDuration').get('value'));
    periodPicker.refreshValue($('cPeriodTime').get('value'));
    startDatePicker.refreshValue($('cStartDate').get('value'));

    var endDateValue = $('cEndDate').get('value');
    if (endDateValue)
        endDatePicker.refreshValue(endDateValue);
    else
        $('cEndDate').set('');

    // Load Data
    timesManager.loadDataDisplay();
    parametersManager.loadDataDisplay();
    displayDataPreview();

    baseValidationManager.reset();
    // same code #details_onSelect end
}

function reset() {
    $('cActionType').selectedIndex = 0;
    $('cScheduleType').selectedIndex = 0;
    $('cParamType').selectedIndex = 0;
    $('cProfiles').selectedIndex = 0;
    $('cExportPeriod').selectedIndex = 0;

    update_parmaWriter_visibility();
    update_action_visibility();
    update_schedule_visibility();
    update_export_period_visibility();
    update_export_variations_visibility();

    $('cFilters').set('value', '');

    filterManager.loadData();
    baseValidationManager.reset();
}

function details_onAdd() {
    reset();

    // Initialize Defaults     
    $('cIsEnabled').checked = true;

    $('cRetryDuration').set('value', '01:00:00');
    retryDurationPicker.refreshValue($('cRetryDuration').get('value'));

    $('cRetryTimeout').set('value', '00:05:00');
    retryTimeoutPicker.refreshValue($('cRetryTimeout').get('value'));

    $('cPeriodInt').set('value', '1');

    $('cHasExportFolder').checked = true;
    $('cExportFolder').set('value', actionScheduler_uiDictionary['defaultDataExportPath']);
    checkExportEnabling();

    timesManager.resetDataEdit();
    parametersManager.resetDataEdit();

    // Command
    $('cActionType').selectedIndex = 0;

    displayDataPreview();
    update_action_visibility();
}

function details_onCancel() {
    // reset();

    // same code #details_onSelect begin
    // Preview
    var previewBtn = document.getElement('div.wkaToolbarArea td.dosome a.watGenericPreview');
    if (previewBtn)
        previewBtn.removeClass('disabled');

    // Update Picker values
    retryTimeoutPicker.refreshValue($('cRetryTimeout').get('value'));
    retryDurationPicker.refreshValue($('cRetryDuration').get('value'));
    periodPicker.refreshValue($('cPeriodTime').get('value'));
    startDatePicker.refreshValue($('cStartDate').get('value'));

    var endDateValue = $('cEndDate').get('value');
    if (endDateValue)
        endDatePicker.refreshValue(endDateValue);
    else
        $('cEndDate').set('');

    // Load Data
    timesManager.loadDataDisplay();
    parametersManager.loadDataDisplay();
    displayDataPreview();

    baseValidationManager.reset();

    // same code #details_onSelect end
}

function details_onSave(event) {
    $('cTimes').set('value', timesManager.saveData());
    $('cParam').set('value', parametersManager.saveData());
 
    // In case we manually clean the date value to '' the value of cStartDate does not become equal to '', it stil stores to previuos value
    // so we need to adjust values of date picker before submit
    if (!startDatePicker.visual.value)
        $('cStartDate').set('value', '');
    if (!endDatePicker.visual.value)
        $('cEndDate').set('value', '');


    var baseValidation = true;
    var nameValidation = vcName.validate();

    // Validate Retry
    var scheduleSelectedIndex = $('cScheduleType').selectedIndex;
    var retryValidation;
    if (scheduleSelectedIndex == 0) { // Periodic
        retryValidation = vcPeriodTime.validate();
        baseValidation = baseValidationManager.validate();
    }
    else if (scheduleSelectedIndex == 1) { // Daily
        retryValidation = vcPeriodInt.validate() && vcTimesList.validate();
        baseValidation = baseValidationManager.validate();
    }
    else if (scheduleSelectedIndex == 2) { // Weekly
        retryValidation = vcPeriodInt.validate() && vcTimesList.validate();
        baseValidation = baseValidationManager.validate();
    } else { // On Demand
        retryValidation = true;
    }

    // Validate Action
    var actionValidation = false;
    var actionSelectedIndex = $('cActionType').selectedIndex;
    if (actionSelectedIndex == 0) {
        // Command
        actionValidation = vcCommand.validate();
    }
    else if (actionSelectedIndex == 1) {
        // Write parameters
        var paramSelectedIndex = $('cParamType').selectedIndex;
        if (paramSelectedIndex == 0) {
            // File Map
            actionValidation = vcMapFileName.validate();
        } else {
            // Param Write
            actionValidation = vcWritingList.validate();
        }
    }
    else if (actionSelectedIndex == 2) {
        // Data export
        actionValidation = vcExportEmail.validate();
    }
    if (baseValidation && retryValidation && actionValidation && nameValidation)
        myClientDataEntry.exeToolbarButton(event);

    return true;
}

function details_onEdit() {
    // Load Data
    timesManager.loadDataEdit();
    parametersManager.loadDataEdit();
    commandArgumentsManager.loadDataEdit();
    displayDataPreview();
    checkExportEnabling();
}

function onlyDigits(e) {
    if (e.event.keyCode == 46 || e.event.keyCode == 8 || e.event.keyCode == 37 || e.event.keyCode == 39) {
        return true;
    }
    if (e.event.keyCode < 95) {
        if (e.event.keyCode < 48 || e.event.keyCode > 57) {
            e.stop();
            return false;
        }
    }
    else {
        if (e.event.keyCode < 96 || e.event.keyCode > 105) {
            e.stop();
            return false;
        }
    }
}



function checkExportEnabling() {
    $('cExportFolder').set('disabled', ($('cHasExportFolder').checked ? '' : 'disabled'));
    $('cExportEmailRecipients').set('disabled', ($('cHasExportEmailRecipients').checked ? '' : 'disabled'));
}

function getXMLFilters() {
    var xmlDoc;
    var txt = $('cFilters').get('value');
    if (txt == "") {
        txt = "<filter culture=\"en-gb\"><selector/></filter>"; // Base filter
        $('cFilters').set('value', txt);
    }
    if (window.DOMParser) { // No IE
        parser = new DOMParser();
        xmlDoc = parser.parseFromString(txt, "text/xml");
    } else // Internet Explorer
    {
        xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
        xmlDoc.async = "false";
        xmlDoc.loadXML(txt);
    }
    return xmlDoc;
}

function displayDataPreview(culture) {
    if ($defined(culture)) $('cCulture').set('html', culture); else $('cCulture').set('html', '');
    previewTree.clear();
    var xmlDoc = getXMLFilters();
    var selectors = xmlDoc.getElementsByTagName("selector");
    for (var selectorIndex = 0; selectorIndex < selectors.length; selectorIndex++) {

        var selectorNode = new Jx.TreeFolder({
            label: filterManager.nodesDictionary["selector"],
            image: '/media/img/Others/ActionSheduler/selector.gif',
            imageClass: 'simpleFolder',
            open: true
        });
        filterManager.attachAttributesTree(selectorNode, selectors[selectorIndex]);
        // Insert Resx Types
        var resxNode = selectors[selectorIndex].getElementsByTagName('resourceTypes');
        var resourceTypes = "";
        if (resxNode.length > 0) {
            if (document.all) {
                resourceTypes = resxNode[0].text;
            } else {
                resourceTypes = resxNode[0].textContent;
            }
        }

        if (resourceTypes != "") {
            var item = new Jx.TreeItem({
                image: '/media/img/Others/ActionSheduler/attribute.gif',
                label: filterManager.nodesDictionary["resourceTypes"] + '=' + resourceTypes
            });
            selectorNode.append(item);
        }

        var interfaces = selectors[selectorIndex].getElementsByTagName("interface");

        for (var interfaceIndex = 0; interfaceIndex < interfaces.length; interfaceIndex++) {
            var interfaceNode = new Jx.TreeFolder({
                label: filterManager.nodesDictionary["interface"],
                image: '/media/img/Others/ActionSheduler/interface.gif',
                imageClass: 'simpleFolder',
                open: true
            });
            selectorNode.append(interfaceNode);
            filterManager.attachAttributesTree(interfaceNode, interfaces[interfaceIndex]);
            var devices = interfaces[interfaceIndex].getElementsByTagName("device");
            for (var deviceIndex = 0; deviceIndex < devices.length; deviceIndex++) {
                var deviceNode = new Jx.TreeFolder({
                    label: filterManager.nodesDictionary["device"],
                    image: '/media/img/Others/ActionSheduler/device.gif',
                    imageClass: 'simpleFolder',
                    open: true
                });
                filterManager.attachAttributesTree(deviceNode, devices[deviceIndex]);
                interfaceNode.append(deviceNode);

                var resources = devices[deviceIndex].getElementsByTagName("resource");
                for (var resourceIndex = 0; resourceIndex < resources.length; resourceIndex++) {
                    var resourceNode = new Jx.TreeFolder({
                        label: filterManager.nodesDictionary["resource"],
                        image: '/media/img/Others/ActionSheduler/resource.gif',
                        imageClass: 'simpleFolder',
                        open: true
                    });
                    filterManager.attachAttributesTree(resourceNode, resources[resourceIndex]);
                    deviceNode.append(resourceNode);
                }
            }
        }
        previewTree.append(selectorNode);
    }
}

window.addEvent('domready', function () {

    Element.implement({
        show: function () { return this.setStyle('display', ''); },
        hide: function () { return this.setStyle('display', 'none'); }
    });

    window.periodicSchedulationOption = $$('#cScheduleType option[value="1"]')[0];
    window.dailySchedulationOption = $$('#cScheduleType option[value="2"]')[0];
    window.weeklySchedulationOption = $$('#cScheduleType option[value="3"]')[0];
    window.onDemandSchedulationOption = $$('#cScheduleType option[value="4"]')[0];

    if (Browser.ie == true && Browser.version < 9) {
        $('ToolBar1$ilbEdit').addEvent('click', details_onEdit);
        $('ToolBar1$ilbAdd').addEvent('click', details_onAdd);
        $('ToolBar1$ilbCancel').addEvent('click', details_onCancel);
    }

    myClientDataEntry = new clientDataEntry('list');

    if (Browser.ie == null || (Browser.ie == true && Browser.version >= 9)) {
        $('ToolBar1$ilbEdit').addEvent('click', details_onEdit);
        $('ToolBar1$ilbAdd').addEvent('click', details_onAdd);
        $('ToolBar1$ilbCancel').addEvent('click', details_onCancel);
    }

    $('ToolBar1$ilbSave').removeEvents();
    $('ToolBar1$ilbSave').addEvent('click', details_onSave);

    var previewBtn = document.getElement('div.wkaToolbarArea td.dosome a.watGenericPreview');
    if (previewBtn) {
        previewBtn.addClass('disabled');
        previewBtn.addEvent('click', openPreview);
        previewBtn.onclick = function () { return false; };
    }

    for (var i = 0; i < myClientDataEntry.cdeRecordListItemStores.length; i++) {
        var crow = myClientDataEntry.cdeRecordListItemStores[i].getFirstParentByTag('tr');
        var clnk = crow.getElement('a');
        clnk.removeEvents();
        clnk.addEvent('click', details_onSelect);
    }

    $('cHasExportFolder').addEvent('click', checkExportEnabling);
    $('cHasExportEmailRecipients').addEvent('click', checkExportEnabling);

    $('cRetryTimeout').setStyle('width', '90px');
    $('cRetryDuration').setStyle('width', '90px');
    $('cPeriodTime').setStyle('width', '90px');
    $('cStartDate').setStyle('width', '160px');
    $('cEndDate').setStyle('width', '160px');
    $('cPeriodInt').setStyle('width', '80px');
    $('cPeriodInt').addEvents({
        'keydown': onlyDigits
    });
    retryTimeoutPicker = new TimeSpanPicker('cRetryTimeout',
        {
            pickerClass: 'datepicker_vista',
            allowEmpty: true,
            titleMessage: actionScheduler_uiDictionary["ts_SetMessage"],
            formatMessage: actionScheduler_uiDictionary["ts_FormatMessage"],
            confirmText: actionScheduler_uiDictionary["ts_confirmText"],
            debug: false
        });

    retryDurationPicker = new TimeSpanPicker('cRetryDuration',
        {
            pickerClass: 'datepicker_vista',
            allowEmpty: true,
            titleMessage: actionScheduler_uiDictionary["ts_SetMessage"],
            formatMessage: actionScheduler_uiDictionary["ts_FormatMessage"],
            confirmText: actionScheduler_uiDictionary["ts_confirmText"],
            debug: false
        });

    periodPicker = new TimeSpanPicker('cPeriodTime',
        {
            pickerClass: 'datepicker_vista',
            allowEmpty: true,
            titleMessage: actionScheduler_uiDictionary["ts_SetMessage"],
            formatMessage: actionScheduler_uiDictionary["ts_FormatMessage"],
            confirmText: actionScheduler_uiDictionary["ts_confirmText"],
            debug: false
        });

    startDatePicker = new DatePicker('cStartDate', {
        pickerClass: 'datepicker_vista',
        timePicker: true,
        inputOutputFormat: 'd-m-Y @ H:i',
        format: 'd-m-Y @ H:i',
        allowEmpty: true,
        days: global_daysOfWeekDictionary,
        months: global_monthsDictionary,
        selectTimeMessage: actionScheduler_uiDictionary["date_selectTimeMessage"],
        confirmText: actionScheduler_uiDictionary["date_ok"],
        debug: false
    });

    endDatePicker = new DatePicker('cEndDate', {
        pickerClass: 'datepicker_vista',
        timePicker: true,
        inputOutputFormat: 'd-m-Y @ H:i',
        format: 'd-m-Y @ H:i',
        allowEmpty: true,
        days: global_daysOfWeekDictionary,
        months: global_monthsDictionary,
        selectTimeMessage: actionScheduler_uiDictionary["date_selectTimeMessage"],
        confirmText: actionScheduler_uiDictionary["date_ok"],
        debug: false
    });

    timesManager.initialize();
    parametersManager.initialize();
    commandArgumentsManager.initialize();

    update_schedule_visibility();
    update_export_period_visibility();
    update_action_visibility();
    update_parmaWriter_visibility();
    update_export_variations_visibility();

    $('cScheduleType').addEvent('change', update_schedule_visibility);
    $('cExportPeriod').addEvent('change', update_export_period_visibility);
    $('cActionType').addEvent('change', update_action_visibility);
    $('cParamType').addEvent('change', update_parmaWriter_visibility);
    $('cExportUndersampling').addEvent('change', update_export_variations_visibility);

    baseValidationManager.initialize();
    baseValidationManager.addControl('cRetryTimeout', '', true, null);
    baseValidationManager.addControl('cRetryDuration', '', true, null);
    baseValidationManager.addControl('cStartDate', '', true, null);
    // baseValidationManager.addControl('cEndDate', '', true, null);
    // baseValidationManager.addControl('cName', '', true, null);

    vcName = new ValidationControl('cName', '', true, null);

    vcPeriodTime = new ValidationControl('cPeriodTime', '', true, null);
    vcPeriodInt = new ValidationControl('cPeriodInt', '', true, null);
    vcTimesList = new ValidationControl('time_emptyMessage', '', false, function () { return $('times_message_panel').style.display == 'none'; });

    vcCommand = new ValidationControl('decCommand', '', true, null);
    // A validation for command arguments should be introduced.    

    vcMapFileName = new ValidationControl('cParamFileName', '', true, null);
    vcWritingList = new ValidationControl('param_emptyMessage', '', false, function () { return $('parameters_message_panel').style.display == 'none'; });

    vcExportEmail = new ValidationControl('cExportEmailRecipients', '', false,
        function () {
            return !$('cHasExportEmailRecipients').checked ||
                $('cExportEmailRecipients').get('value').trim() != "";
        });

    $('cFilters').set('value', '');
    this.previewTree = new Jx.Tree({ parent: 'treeFilterArea' });


    if ($('cTimes').get('value')) {
        // there are values to fill the times grid !! (this can be a postback, so the correct state is restored on postback!)
        // this fix the issue in code-behind of and impossibile validation server side.
        var isDisabled = $('cName').getAttribute("disabled");
        if (isDisabled) {
            timesManager.loadDataDisplay();
        }
        else {
            timesManager.loadDataEdit();
        }
    }

});