﻿window.addEvent('domready', function() {
    Element.implement({
        show: function() { return this.setStyle('display', ''); },
        hide: function() { return this.setStyle('display', 'none'); }
    });

    filterManager.initialize(global_selectorTypes, global_resourceTypes, global_interfaceAttributeArray, global_deviceAttributeArray, global_interfaceArray, global_uiDictionary, global_nodesDictionary);
});

/* -----------------------------------------------------------------------------|
| F I L T E R   M A N A G E R
\----------------------------------------------------------------------------- */

var filterManager = {
    initialize: function(selectorTypes, resourceTypes, interfaceAttributes, deviceAttributes, interfaces, uiDictionary, nodesDictionary) {
        //filterManager.previewTree = new Jx.Tree({ parent: 'treeFilterArea' });
        this.cultures = $(document.body).getElements('select.adaptiveFilterPopupCultureDdl');

        this.localizeNodes = false;
        this.nodesDictionary = nodesDictionary;
        this.uiDictionary = uiDictionary;

        this.editList = $('nodeList');
        this.nodeArray = new Array();

        filterManager.localizeUI(uiDictionary, nodesDictionary);

        // filterManager.loadData();
        filterManager.hideAll();

        filterManager.initializeSelectorDDL(selectorTypes);
        filterManager.initializeResourceDDL(resourceTypes);
        filterManager.initializeInterfaceAttributeDDL(interfaceAttributes);
        filterManager.initializeDeviceAttributeDDL(deviceAttributes);
        filterManager.initializeDeviceInterfacesDDL(interfaces);

        $('btnPopUpFiltersClose').addEvent('click', filterManager.closeModalPopUp);
        $('btnPopUpFiltersSave').addEvent('click', filterManager.saveFilters);
        $('btnPopUpFiltersExit').addEvent('click', filterManager.closeModalPopUp);
        $('btnPopUpFiltersCopyFromDefault').addEvent('click', filterManager.copyFromDefault);
        $('btnPopUpFiltersCopyFromDevice').addEvent('click', filterManager.copyFromDevice);

        $('btn_add_selector').addEvent('click', filterManager.addNodeSelector);
        $('btn_save_selector').addEvent('click', filterManager.saveNodeSelector);
        $('btn_save_interface').addEvent('click', filterManager.saveNodeInterface);
        $('btn_save_device').addEvent('click', filterManager.saveNodeDevice);
        $('btn_save_resource').addEvent('click', filterManager.saveNodeResource);

        $('ddl_attr_interface').addEvent('change', filterManager.attributeInterfaceChange);
        $('ddl_attr_device').addEvent('change', filterManager.attributeDeviceChange);
        $('ddl_interface_device').addEvent('change', filterManager.attributeDeviceChange);

        $('btn_set_device').addEvent('click', filterManager.setDeviceValue);
        $('btn_set_interface').addEvent('click', filterManager.setInterfaceValue);

        $('btn_set_device').set('disabled', 1);
        $('btn_set_interface').set('disabled', 1);

        $('val_id_resource').hide();
        $('val_id_interface').hide();
        $('val_name_interface').hide();
        $('val_address_device').hide();

        $('pl_icon_quick_interface').hide();
        $('pl_icon_quick_device').hide();
    },

    attributeInterfaceChange: function() {
        var ddl = $('ddl_attr_interface');
        var currentAttribute = ddl.get('value');
        if (currentAttribute == '') {
            $('ddl_value_interface').options.length = 0;
            $('btn_set_interface').set('disabled', 1);
            return;
        }
        // Call Option Update
        $('pl_icon_quick_interface').show();

        var param = "modality=ajax" + "&exec=getInterfaces" + "&argumentID=" + currentAttribute;
        new Request(
            {
                url: '/SchedulerActionUtilityInvocation.rix',
                onComplete: function(request) {
                    var list = JSON.decode(request);
                    if (list.length > 0) {
                        filterManager.addOptionsToSelect('ddl_value_interface', list, true);
                        $('btn_set_interface').set('disabled', 0);
                    } else {
                        filterManager.addOptionsToSelect('ddl_value_interface', [{ id: '', display: uiDictionary["nodata"] }], true);
                        $('btn_set_interface').set('disabled', 1);
                    }
                    filterManager.addOptionsToSelect('ddl_value_interface', list, true);
                    $('btn_set_interface').set('disabled', 0);
                    $('pl_icon_quick_interface').hide();
                },
                timeout: 5000
            }).send(param);
    },

    attributeDeviceChange: function() {
        var ddlAttr = $('ddl_attr_device');
        var ddlInterface = $('ddl_interface_device');
        var currentAttribute = ddlAttr.get('value');
        var currentInterface = ddlInterface.get('value');

        if (currentAttribute == '' || currentInterface == '') {
            $('ddl_value_device').options.length = 0;
            $('btn_set_device').set('disabled', 1);
            return;
        }
        $('pl_icon_quick_device').show();
        var param = "modality=ajax" + "&exec=getDevices" + "&argumentID=" + currentAttribute + "&interfaceID=" + currentInterface;
        new Request(
            {
                url: '/SchedulerActionUtilityInvocation.rix',
                onComplete: function(request) {
                    var list = JSON.decode(request);
                    if (list.length > 0) {
                        filterManager.addOptionsToSelect('ddl_value_device', list, true);
                        $('btn_set_device').set('disabled', 0);
                    } else {
                        filterManager.addOptionsToSelect('ddl_value_device', [{ id: '', display: uiDictionary["nodata"] }], true);
                        $('btn_set_device').set('disabled', 1);
                    }
                    $('pl_icon_quick_device').hide();
                },
                timeout: 5000
            }).send(param);
    },

    setDeviceValue: function() {
        var destAttr = $('ddl_attr_device').get('value');
        var value = $('ddl_value_device').get('value');
        switch (destAttr) {
        case "name":
            $('filter_device_name').set('value', value);
            break;
        case "model":
            $('filter_device_model').set('value', value);
            break;
        case "address":
            $('filter_device_address').set('value', value);
            break;
        default:
            break;
        }
    },

    setInterfaceValue: function() {
        var destAttr = $('ddl_attr_interface').get('value');
        var value = $('ddl_value_interface').get('value');
        switch (destAttr) {
        case "id":
            $('filter_interface_id').set('value', value);
            break;
        case "name":
            $('filter_interface_name').set('value', value);
            break;
        default:
            break;
        }
    },

    localizeUI: function(uiDictionary, nodesDictionary) {
        $('btn_add_selector').set('html', uiDictionary["addSelector"]);

        $('btn_save_selector').set('value', uiDictionary["save"]);
        $('btn_save_interface').set('value', uiDictionary["save"]);
        $('btn_save_device').set('value', uiDictionary["save"]);
        $('btn_save_resource').set('value', uiDictionary["save"]);

        $('btn_set_device').set('value', uiDictionary["setvalue"]);
        $('btn_set_interface').set('value', uiDictionary["setvalue"]);

        $('title_filterSelector').set('html', uiDictionary["filter"]);
        $('title_filterInterface').set('html', uiDictionary["filter"]);
        $('title_filterDevice').set('html', uiDictionary["filter"]);
        $('title_filterResource').set('html', uiDictionary["filter"]);

        $('title_quickSetDevice').set('html', uiDictionary["quickSet"]);
        $('title_quickSetInterface').set('html', uiDictionary["quickSet"]);

        $('title_attributeInterface').set('html', uiDictionary["attribute"]);
        $('title_valueInterface').set('html', uiDictionary["value"]);

        $('title_attributeDevice').set('html', uiDictionary["attribute"]);
        $('title_interfaceDevice').set('html', uiDictionary["interface"]);
        $('title_valueDevice').set('html', uiDictionary["value"]);
        $('title_filterList').set('html', uiDictionary["filterlist"]);

        $('val_id_resource').set('alt', uiDictionary["tooltip_validation_resource_id"]);
        $('val_id_interface').set('alt', uiDictionary["tooltip_validation_interface_id"]);
        $('val_name_interface').set('alt', uiDictionary["tooltip_validation_interface_name"]);
        $('val_address_device').set('alt', uiDictionary["tooltip_validation_device_address"]);
        $('val_id_resource').set('title', uiDictionary["tooltip_validation_resource_id"]);
        $('val_id_interface').set('title', uiDictionary["tooltip_validation_interface_id"]);
        $('val_address_device').set('title', uiDictionary["tooltip_validation_device_address"]);

        $('label_type_selector').set('html', nodesDictionary["type"]);
        $('label_resources_selector').set('html', nodesDictionary["resources"]);
        $('label_id_interface').set('html', nodesDictionary["id"]);
        $('label_name_interface').set('html', nodesDictionary["name"]);
        $('label_address_device').set('html', nodesDictionary["address"]);
        $('label_name_device').set('html', nodesDictionary["name"]);
        $('label_model_device').set('html', nodesDictionary["model"]);
        $('label_id_resource').set('html', nodesDictionary["id"]);
        $('label_name_resource').set('html', nodesDictionary["name"]);
        $('label_type_resource').set('html', nodesDictionary["type"]);
    },

    addOptionsToSelect: function(selectId, options, clearBeforeInsert) {
        var elSel = $(selectId);
        if (clearBeforeInsert)
            elSel.options.length = 0;
        for (var index = 0; index < options.length; index++) {
            var elOptNew = document.createElement('option');
            elOptNew.text = options[index].display;
            elOptNew.value = options[index].id;
            try {
                elSel.add(elOptNew, null);
            } catch(ex) {
                elSel.add(elOptNew);
            }
        }
    },

    initializeInterfaceAttributeDDL: function(attributes) {
        filterManager.addOptionsToSelect('ddl_attr_interface', [{ id: '', display: filterManager.uiDictionary["select"] }], false);
        filterManager.addOptionsToSelect('ddl_attr_interface', attributes, false);
    },

    initializeDeviceAttributeDDL: function(attributes) {
        filterManager.addOptionsToSelect('ddl_attr_device', [{ id: '', display: filterManager.uiDictionary["select"] }], false);
        filterManager.addOptionsToSelect('ddl_attr_device', attributes, false);
    },

    initializeDeviceInterfacesDDL: function(interfaces) {
        filterManager.addOptionsToSelect('ddl_interface_device', [{ id: '', display: filterManager.uiDictionary["select"] }], false);
        filterManager.addOptionsToSelect('ddl_interface_device', interfaces, false);
    },

    initializeResourceDDL: function(resourceTypes) {
        filterManager.addOptionsToSelect('ddl_type_resource', resourceTypes, false);
    },

    initializeSelectorDDL: function(selectorTypes) {
        filterManager.addOptionsToSelect('ddl_type_selector', selectorTypes, false);
    },

    loadData: function() {
        //filterManager.getXMLFilters();
        // filterManager.displayDataPreview();
    },

    getAttributesString: function(xmlNode) {
        var dataString = "";
        for (var index = 0; index < xmlNode.attributes.length; index++) {
            var att = xmlNode.attributes[index];
            if (filterManager.nodesDictionary[att.name] != null)
                dataString += filterManager.nodesDictionary[att.name] + '="' + att.value + '" ';
            else
                dataString += att.name + '="' + att.value + '" ';
        }
        return dataString;
    },

    hideAll: function() {
        $('selectorEdit').hide();
        $('interfaceEdit').hide();
        $('deviceEdit').hide();
        $('resourceEdit').hide();
    },

    switchUI: function(tagName) {
        filterManager.hideAll();
        switch (tagName) {
        case "selector":
            $('selectorEdit').show();
            break;
        case "interface":
            $('interfaceEdit').show();
            break;
        case "device":
            $('deviceEdit').show();
            break;
        case "resource":
            $('resourceEdit').show();
            break;
        default:
        }
    },

    setAttribute: function(node, attributeName, value) {
        var attribute = node.getAttributeNode(attributeName);
        if (value == "" && attribute != null) {
            node.removeAttribute(attributeName);
            return;
        }
        if (value == "")
            return;
        if (attribute == null) {
            attribute = filterManager.xmlDoc.createAttribute(attributeName);
            attribute.nodeValue = value;
            node.setAttributeNode(attribute);
        } else {
            attribute.nodeValue = value;
        }
    },

    getAttribute: function(node, attributeName) {
        var attribute = node.getAttribute(attributeName);
        if (attribute != null)
            return attribute;
        return '';
    },

    saveNodeSelector: function() {
        var node = filterManager.currentNode;

        var selectorType = $('ddl_type_selector').get('value');
        if (selectorType == "") selectorType = "add";
        filterManager.setAttribute(node, 'type', selectorType);

        var checkBoxs = $('checkboxs_resource_selector').getElements('input');

        var nodeValue = "";

        checkBoxs.each(function(control) {
            if (control.checked)
                nodeValue = nodeValue + control.get('value') + ' ';
        });

        var resxNode = node.getElementsByTagName('resourceTypes');
        if (nodeValue.length > 0) {
            nodeValue = nodeValue.substring(0, nodeValue.length - 1);
            if (resxNode.length <= 0) {
                var newNode = filterManager.xmlDoc.createElement('resourceTypes');
                if (document.all) {
                    newNode.text = nodeValue;
                } else {
                    newNode.textContent = nodeValue;
                }
                node.appendChild(newNode);
            } else {
                if (document.all) {
                    resxNode[0].text = nodeValue;
                } else {
                    resxNode[0].textContent = nodeValue;
                }
            }
        } else {
            if (resxNode.length > 0)
                node.removeChild(resxNode[0]);
        }

        filterManager.hideAll();
        filterManager.refreshEditList();
    },

    saveNodeInterface: function() {
        var interfaceId = $('filter_interface_id').get('value');
        var re;
        if (interfaceId != "") {
            re = new RegExp("^\\s*(\\*|\\d+)\\s*$");
            if (!interfaceId.match(re)) {
                $('val_id_interface').show();
                return;
            }
        }
        $('val_id_interface').hide();

        var interfaceName = $('filter_interface_name').get('value');
        if (interfaceName != "") {
            re = new RegExp("^(\\s*([L|l]ogical)|([A|a]lgorithms)|(\\*|COM([\\d\\?]+|\\*)|((\\*|[2\\?][5\\?][0-5\\?]|[2\\?][0-4\\?][\\d\\?]|[0-1\\?][\\d\\?][\\d\\?]|[\\d\\?][\\d\\?]|[\\d\\?])(\\.(\\*|[2\\?][5\\?][0-5\\?]|[2\\?][0-4\\?][\\d\\?]|[0-1\\?][\\d\\?][\\d\\?]|[\\d\\?][\\d\\?]|[\\d\\?])){3})(:[\\d\\?]{1,5})?)\\s*)$");
            if (!interfaceName.match(re)) {
                $('val_name_interface').show();
                return;
            }
        }
        $('val_name_interface').hide();

        var node = filterManager.currentNode;
        filterManager.setAttribute(node, 'id', interfaceId);
        filterManager.setAttribute(node, 'name', $('filter_interface_name').get('value'));

        filterManager.hideAll();
        filterManager.refreshEditList();
    },

    saveNodeDevice: function() {
        var address = $('filter_device_address').get('value');
        if (address != "") {
            var re = new RegExp("^\\s*(\\*|([0\\?]?[\\d\\?]|[1\\?][0-5\\?]|\\*):([0\\?]?[\\d\\?]|[1\\?][0-5\\?]|\\*))\\s*$");
            if (!address.match(re)) {
                $('val_address_device').show();
                return;
            }
        }
        $('val_address_device').hide();
        var node = filterManager.currentNode;
        filterManager.setAttribute(node, 'name', $('filter_device_name').get('value'));
        filterManager.setAttribute(node, 'model', $('filter_device_model').get('value'));
        filterManager.setAttribute(node, 'address', address);

        filterManager.hideAll();
        filterManager.refreshEditList();
    },

    saveNodeResource: function() {
        var resourceId = $('filter_resource_id').get('value');
        if (resourceId != "") {
            var re = new RegExp("^\\s*(\\*|([A-Z\\?]{3}|\\*)([\\d\\?]{5}|\\*)(:([\\d\\?]+|\\*))?)(-(.+|\\*))?\\s*$");
            if (!resourceId.match(re)) {
                $('val_id_resource').show();
                return;
            }
        }
        $('val_id_resource').hide();
        var node = filterManager.currentNode;
        filterManager.setAttribute(node, 'id', resourceId);
        filterManager.setAttribute(node, 'name', $('filter_resource_name').get('value'));
        filterManager.setAttribute(node, 'type', $('ddl_type_resource').get('value'));

        filterManager.hideAll();
        filterManager.refreshEditList();
    },

    loadNodeSelector: function(node) {
        // Load Type
        $('ddl_type_selector').set('value', filterManager.getAttribute(node, 'type'));
        // Load Resxs
        var resxNode = node.getElementsByTagName('resourceTypes');
        var checkBoxs = $('checkboxs_resource_selector').getElements('input');
        if (resxNode.length > 0) {
            var values = "";
            if (document.all) {
                values = resxNode[0].text;
            } else {
                values = resxNode[0].textContent;
            }

            var splitValues = values.split(' ');
            if (splitValues.length > 0) {
                checkBoxs.each(function(control) {
                    if (splitValues.contains('all') || splitValues.contains(control.get('value')))
                        control.checked = true;
                    else
                        control.checked = false;
                });
            }
        } else {
            checkBoxs.each(function(control) {
                control.checked = false;
            });
        }
    },

    loadNodeInterface: function(node) {
        $('filter_interface_id').set('value', filterManager.getAttribute(node, 'id'));
        $('filter_interface_name').set('value', filterManager.getAttribute(node, 'name'));
    },

    loadNodeDevice: function(node) {
        $('filter_device_model').set('value', filterManager.getAttribute(node, 'model'));
        $('filter_device_address').set('value', filterManager.getAttribute(node, 'address'));
        $('filter_device_name').set('value', filterManager.getAttribute(node, 'name'));
    },

    loadNodeResource: function(node) {
        $('filter_resource_id').set('value', filterManager.getAttribute(node, 'id'));
        $('filter_resource_name').set('value', filterManager.getAttribute(node, 'name'));
        $('ddl_type_resource').set('value', filterManager.getAttribute(node, 'type'));
    },

    editNode: function(event, nodeId) {
        filterManager.currentNode = filterManager.nodeArray[nodeId];
        filterManager.switchUI(filterManager.currentNode.tagName);
        switch (filterManager.currentNode.tagName) {
        case "selector":
            filterManager.loadNodeSelector(filterManager.currentNode);
            break;
        case "interface":
            filterManager.loadNodeInterface(filterManager.currentNode);
            break;
        case "device":
            filterManager.loadNodeDevice(filterManager.currentNode);
            break;
        case "resource":
            filterManager.loadNodeResource(filterManager.currentNode);
            break;
        default:
        }
        event.preventDefault();
        return false;
    },

    removeNode: function(event, nodeId) {
        var nodeToRemove = filterManager.nodeArray[nodeId];
        var partentNode = nodeToRemove.parentNode;
        partentNode.removeChild(nodeToRemove);
        filterManager.refreshEditList();
        event.preventDefault();
        return false;
    },

    addNodeSelector: function() {
        var newel = filterManager.xmlDoc.createElement("selector");
        filterManager.xmlDoc.documentElement.appendChild(newel);
        filterManager.refreshEditList();
    },

    addNode: function(event, nodeId) {
        var nodeToExtend = filterManager.nodeArray[nodeId];
        var nodeName = "";
        switch (nodeToExtend.tagName) {
        case "selector":
            nodeName = "interface";
            break;
        case "interface":
            nodeName = "device";
            break;
        case "device":
            nodeName = "resource";
            break;
        default:
        }
        var newel = filterManager.xmlDoc.createElement(nodeName);
        nodeToExtend.appendChild(newel);
        filterManager.refreshEditList();
        event.preventDefault();
        return false;
    },

    moveUpNode: function(event, nodeId) {
        var nodeToMove = filterManager.nodeArray[nodeId];

        if (nodeToMove.previousSibling != null) {
            nodeToMove.parentNode.insertBefore(nodeToMove, nodeToMove.previousSibling);
            filterManager.refreshEditList();
        }
        event.preventDefault();
        return false;
    },

    moveDownNode: function(event, nodeId) {
        var nodeToMove = filterManager.nodeArray[nodeId];

        if (nodeToMove.nextSibling != null) {
            nodeToMove.parentNode.insertBefore(nodeToMove.nextSibling, nodeToMove);
            filterManager.refreshEditList();
        }
        event.preventDefault();
        return false;
    },

    unmarkElement: function(element) {
        element.removeClass('nodeSelected');
    },

    markElement: function(element) {
        element.addClass('nodeSelected');
    },

    addButtons: function(list, nodeId, showAdd, showUpDown, divElement, showButtons) {
        var btnD = new Element('div', { 'class': 'filter_buttons' }).inject(list);

        if (showButtons) {
            if (showUpDown) {
                var btn_up = new Element('a', { 'class': 'filter_button filter_up', href: '#', title: filterManager.uiDictionary["tooltip_node_up"] }).inject(btnD);
                btn_up.addEvents({
                    click: function(event) { filterManager.moveUpNode(event, nodeId); },
                    mouseover: function() { filterManager.markElement(divElement); },
                    mouseleave: function() { filterManager.unmarkElement(divElement); }
                });

                var btnDown = new Element('a', { 'class': 'filter_button filter_down', href: '#', title: filterManager.uiDictionary["tooltip_node_down"] }).inject(btnD);
                btnDown.addEvents({
                    click: function(event) { filterManager.moveDownNode(event, nodeId); },
                    mouseover: function() { filterManager.markElement(divElement); },
                    mouseleave: function() { filterManager.unmarkElement(divElement); }
                });
            }

            if (showAdd) {
                var btnAdd = new Element('a', { 'class': 'filter_button filter_add', href: '#', title: filterManager.uiDictionary["tooltip_node_add"] }).inject(btnD);
                btnAdd.addEvents({
                    click: function(event) { filterManager.addNode(event, nodeId); },
                    mouseover: function() { filterManager.markElement(divElement); },
                    mouseleave: function() { filterManager.unmarkElement(divElement); }
                });
            }

            var btnEdit = new Element('a', { 'class': 'filter_button filter_edit', href: '#', title: filterManager.uiDictionary["tooltip_node_edit"] }).inject(btnD);
            btnEdit.addEvents({
                click: function(event) { filterManager.editNode(event, nodeId); },
                mouseover: function() { filterManager.markElement(divElement); },
                mouseleave: function() { filterManager.unmarkElement(divElement); }
            });

            var btnRemove = new Element('a', { 'class': 'filter_button filter_remove', href: '#', title: filterManager.uiDictionary["tooltip_node_remove"] }).inject(btnD);
            btnRemove.addEvents({
                click: function(event) { filterManager.removeNode(event, nodeId); },
                mouseover: function() { filterManager.markElement(divElement); },
                mouseleave: function() { filterManager.unmarkElement(divElement); }
            });
        }
        var clear = new Element('div', { styles: { clear: 'both' } }).inject(list);
    },

    refreshCultureDdl: function() {
        var culture = filterManager.xmlDoc.childNodes[0].attributes["culture"].nodeValue;
        filterManager.cultures.set('value', culture);
    },

    refreshEditList: function() {
        filterManager.nodeArray = new Array();
        filterManager.editList.set('html', '');

        $('btn_add_selector').setStyle("display", filterManager.readOnly ? "none" : "block");
        filterManager.cultures.set("disabled", filterManager.readOnly ? "disabled" : "");

        $(document.body).getElements('div.rightCol').setStyle("display", filterManager.readOnly ? "none" : "block");
        $("filterList").toggleClass("filterList_FullWidth", filterManager.readOnly);

        var selectors = filterManager.xmlDoc.getElementsByTagName("selector");
        for (var selectorIndex = 0; selectorIndex < selectors.length; selectorIndex++) {
            var row;
            var nodeId;
            row = new Element('div', { 'class': 'filter_edit_item filter_edit_selector' });
            nodeId = filterManager.nodeArray.length + 1;
            filterManager.nodeArray[nodeId] = selectors[selectorIndex];

            row.set('html', filterManager.nodesDictionary["selector"] + ' ' + filterManager.getAttributesString(selectors[selectorIndex]));
            row.inject(filterManager.editList);

            filterManager.addButtons(filterManager.editList, nodeId, true, true, row, !filterManager.readOnly);
            var interfaces = selectors[selectorIndex].getElementsByTagName("interface");
            for (var interfaceIndex = 0; interfaceIndex < interfaces.length; interfaceIndex++) {
                row = new Element('div', { 'class': 'filter_edit_item filter_edit_interface' });
                row.set('html', filterManager.nodesDictionary["interface"] + ' ' + filterManager.getAttributesString(interfaces[interfaceIndex]));
                row.inject(filterManager.editList);
                nodeId = filterManager.nodeArray.length + 1;
                filterManager.nodeArray[nodeId] = interfaces[interfaceIndex];
                filterManager.addButtons(filterManager.editList, nodeId, true, false, row, !filterManager.readOnly);

                var devices = interfaces[interfaceIndex].getElementsByTagName("device");
                for (var deviceIndex = 0; deviceIndex < devices.length; deviceIndex++) {
                    row = new Element('div', { 'class': 'filter_edit_item filter_edit_device' });
                    row.set('html', filterManager.nodesDictionary["device"] + ' ' + filterManager.getAttributesString(devices[deviceIndex]));
                    nodeId = filterManager.nodeArray.length + 1;
                    filterManager.nodeArray[nodeId] = devices[deviceIndex];
                    row.inject(filterManager.editList);
                    filterManager.addButtons(filterManager.editList, nodeId, true, false, row, !filterManager.readOnly);

                    var resources = devices[deviceIndex].getElementsByTagName("resource");
                    for (var resourceIndex = 0; resourceIndex < resources.length; resourceIndex++) {
                        row = new Element('div', { 'class': 'filter_edit_item filter_edit_resource' });
                        row.set('html', filterManager.nodesDictionary["resource"] + ' ' + filterManager.getAttributesString(resources[resourceIndex]));
                        nodeId = filterManager.nodeArray.length + 1;
                        filterManager.nodeArray[nodeId] = resources[resourceIndex];
                        row.inject(filterManager.editList);
                        filterManager.addButtons(filterManager.editList, nodeId, false, false, row, !filterManager.readOnly);
                    }
                }
            }
        }
    },

    viewData: function(textAreaId, additional, filterType) {
        filterManager.readOnly = true;
        filterManager.textArea = $(textAreaId);
        filterManager.originalText = filterManager.textArea.value;
        filterManager.hasBeenSaved = false;

        // Set title
        var title = filterManager.uiDictionary["titleView"] + " ";
        if (filterType == "r") {
            title += filterManager.uiDictionary["titleResourceFilter"];
        } else {
            /* device filter is the most common usage */
            title += filterManager.uiDictionary["titleDeviceFilter"];
        }
        if (additional) title += " - " + additional;
        $('headerFilters').set('html', title);

        $('btnPopUpFiltersClose').setStyle("display", "none");
        $('btnPopUpFiltersSave').setStyle("display", "none");
        $('btnPopUpFiltersExit').setStyle("display", "inline-block");
        $('btnPopUpFiltersCopyFromDefault').setStyle("display", "none");
        $('btnPopUpFiltersCopyFromDevice').setStyle("display", "none");

        filterManager.getXMLFilters();
        filterManager.refreshEditList();
        filterManager.refreshCultureDdl();
        filterManager.openModalPopUp();
    },

    editData: function(textAreaId, onSave, additional, defaultInput, deviceInput) {
        filterManager.readOnly = false;
        filterManager.textArea = $(textAreaId);
        filterManager.originalText = filterManager.textArea.value;
        filterManager.hasBeenSaved = false;
        filterManager.onSave = onSave;

        // Set title
        var title = filterManager.uiDictionary["titleEdit"] + " " + filterManager.uiDictionary["titleDeviceFilter"];

        if (additional) title += " - " + additional;
        $('headerFilters').set('html', title);

        $('btnPopUpFiltersClose').setStyle("display", "inline-block");
        $('btnPopUpFiltersSave').setStyle("display", "inline-block");
        $('btnPopUpFiltersExit').setStyle("display", "none");
        if (defaultInput != null && $(defaultInput).value.trim() != "") {
            filterManager.defaultContent = $(defaultInput);
            $('btnPopUpFiltersCopyFromDefault').setStyle("display", "inline-block");
        } else {
            $('btnPopUpFiltersCopyFromDefault').setStyle("display", "none");
        }
        if (deviceInput != null && $(deviceInput).value.trim() != "") {
            filterManager.deviceContent = $(deviceInput);
            $('btnPopUpFiltersCopyFromDevice').setStyle("display", "inline-block");
        } else {
            $('btnPopUpFiltersCopyFromDevice').setStyle("display", "none");
        }

        filterManager.getXMLFilters();
        filterManager.refreshEditList();
        filterManager.refreshCultureDdl();
        filterManager.hideAll();
        filterManager.openModalPopUp();
    },

    copyFromDefault: function() {
        var temporary = filterManager.textArea.value;
        filterManager.textArea.setFormElementValue(filterManager.defaultContent.value);
        filterManager.getXMLFilters();
        filterManager.refreshEditList();
        filterManager.refreshCultureDdl();
        filterManager.textArea.setFormElementValue(temporary);
    },

    copyFromDevice: function() {
        var temporary = filterManager.textArea.value;
        filterManager.textArea.setFormElementValue(filterManager.deviceContent.value);
        filterManager.getXMLFilters();
        filterManager.refreshEditList();
        filterManager.refreshCultureDdl();
        filterManager.textArea.setFormElementValue(temporary);
    },

    saveFilters: function() {
        var culture = filterManager.cultures.get('value');
        if (culture == "") culture = "en-GB";
        filterManager.xmlDoc.childNodes[0].attributes["culture"].nodeValue = culture;

        var xml;
        if (filterManager.xmlDoc.xml == null) {            
            xml = (new XMLSerializer()).serializeToString(filterManager.xmlDoc);
        } else {
            xml = filterManager.xmlDoc.xml;
        }
        xml = xml.replace(/[\n\r]/g, '');

        filterManager.textArea.setFormElementValue(xml);
        filterManager.loadData();
        filterManager.hasBeenSaved = true;
        filterManager.closeModalPopUp();
        if (filterManager.onSave && typeof(filterManager.onSave) == "function")
            filterManager.onSave(filterManager.cultures.getSelected()[0].get('text'));
    },

    closeModalPopUp: function() {
        if (!filterManager.hasBeenSaved) {
            // Undos editing.
            filterManager.textArea.setFormElementValue(filterManager.originalText);
        }
        $('modalPopUpContainer').setStyle("display", "none");
        return false;
    },

    openModalPopUp: function() {
        $$('.leftCol').each(function(c) { c.show(); });
        $('modalPopUpContainer').setStyle("display", "block");
        return false;
    },

    attachAttributesTree: function(displayNode, xmlNode) {
        for (var index = 0; index < xmlNode.attributes.length; index++) {
            var att = xmlNode.attributes[index];
            var item = new Jx.TreeItem({
                image: '/media/img/Others/ActionSheduler/attribute.gif',
                label: filterManager.nodesDictionary[att.name] + '=' + att.value
            });
            displayNode.append(item);
        }
    },

    uniqueID: function() {
        var newDate = new Date;
        return newDate.getTime() + '';
    },

    getXMLFilters: function() {
        var txt = filterManager.textArea.value;
        if (txt == "") {
            txt = "<filter culture=\"en-gb\"><selector/></filter>"; // Base filter
            filterManager.textArea.setFormElementValue(txt);
        }
        if (window.DOMParser) { // No IE
            parser = new DOMParser();
            this.xmlDoc = parser.parseFromString(txt, "text/xml");
        } else // Internet Explorer
        {
            this.xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
            this.xmlDoc.async = "false";
            this.xmlDoc.loadXML(txt);
        }
    }
};