﻿var _currentLegend;
var _currentAxis;
var _colorPicker;
var _chart;
var _legendsMap = [];
var _axisGuides = [];
var _markedLegendDivId = null;

var _removedPanels = [];

var _isLegendVisible = true;
var _isStatVisible = true;

var _contextMenuLegend;
var _contextMenuAxis;
var _axis_tb_min;
var _axis_tb_max;
var _axis_sp_step;
var _bands_tb_min;
var _bands_tb_max;
var _bands_sel;
var _axis_min_max_control;
var _axis_bands_control;

var toolBarHeight = 122;
var leftContextMenuMargin = 55;
var chartPeriodSelectorHeight = 40;


// IMPORTANT: this JavaScript USE a global Localization array: charts_uiDictionary

// CONFIGURABLE PARAMETERS
var analogPanelMinHeight = 300;
var digitalPanelMinHeight = 30;

function updateChartData() {
    var postBody = "modality=ajax";

    $('labelCancelingMessage').setStyle('display', 'none');

    new Request({
        url: '/HistoricalChartInvocation.rix',        

        onComplete: function (response) {
            if (document.canceled) return;

            // Resets.
            {
                _removedPanels = [];
                _legendsMap = [];
                _axisGuides = [];
            }

            var hasData = false;

            if (response !== "NODATA") {
                var settings = JSON.decode(response);
                hasData = false;
                if (settings.dataSets.length > 0) {
                    var dp = settings.dataSets[0].dataProvider;
                    for (var dataIndex = 0; dataIndex < dp.length; dataIndex++) {
                        var data = dp[dataIndex];
                        data.date = Date.parse(data.date);
                        hasData = true;
                    }
                }

                // Creation of legends DIVs
                var legendContainers = $('legendContainer');
                legendContainers.empty();
                for (var i = 0; i < settings.panels.length; i++) {
                    var legendDiv = new Element('div', {
                        'class': 'legend-box',
                        'id': settings.panels[i].stockLegend.divId
                    });
                    legendContainers.grab(legendDiv);
                }

                setDivChartSize(settings.panels.length - 1);
            }

            if (hasData)
            {
                $('noDataMessage').setStyle('display', 'none');

                _chart = AmCharts.makeChart("chartdiv", settings, 500);
                _chart.addListener("init", chartInit);
                _chart.addListener("rendered", chartRendered);
                _chart.addListener("zoomed", chartZoomed);

                _contextMenuLegend = $('analog-context-menu');
                _contextMenuAxis = $('axis-context-menu');
                _axis_tb_min = $('axis-tb-min');
                _axis_tb_max = $('axis-tb-max');
                _axis_sp_step = $('axis-lb-step');
                _bands_tb_min = $('bands-tb-min');
                _bands_tb_max = $('bands-tb-max');
                _bands_sel = $('bandsSelect');
                _axis_min_max_control = $('axis-min-max-control');
                _axis_bands_control = $('axis-bands-control');

                $('change-axis-color-button').addEvent('click', changeColorAxis);
                $('change-color-button').addEvent('click', changeColorLegend);
                $('change-axis-minmax-button').addEvent('click', showChangeAxis);
                $('assign-axis-minmax-button').addEvent('click', assignAxis);
                $('hide-show-marker-button').addEvent('click', showHideMarker);
                $('change-axis-actual-button').addEvent('click', setActualAxis);
                $('change-axis-bands-button').addEvent('click', changeBands);
                $('assign-axis-bands-button').addEvent('click', assignBands);
                $('clear-axis-bands-button').addEvent('click', clearBands);

                _axis_tb_min.addEvent('click', preventClick);
                _axis_tb_max.addEvent('click', preventClick);
                _bands_tb_min.addEvent('click', preventClick);
                _bands_tb_max.addEvent('click', preventClick);
                _bands_sel.addEvent('click', preventClick);
                $$('.context-menu').each(function (element) {
                    element.addEvent('click', preventClick);
                });

                _axis_tb_min.addEvent('keydown', assignMinMaxOnEnter);
                _axis_tb_max.addEvent('keydown', assignMinMaxOnEnter);
                _bands_tb_min.addEvent('keydown', assignBandsOnEnter);
                _bands_tb_max.addEvent('keydown', assignBandsOnEnter);

                document.addEvent('click', function () {
                    hideAllMenus();
                }.bind(this));

                setColorPicker();

                $('stat-box').setStyle('display', 'block');
                $('chartdiv').setStyle('display', 'block');
            }
            else
            {
                $('stat-box').setStyle('display', 'none');
                $('chartdiv').setStyle('display', 'none');
                $('noDataMessage').setStyle('display', 'block');

                if (typeof (window.hideWaitModal !== "undefined"))
                    window.hideWaitModal();
            }
        },
        timeout: 500000
    }).send(postBody);
}

window.addEvent('domready', function () {
    document.canceled = false;
});

function setColorPicker() {
    if (!_colorPicker) {
        _colorPicker = new MooRainbow('axis-context-menu',
            {
                startColor: [255, 255, 255],
                imgPath: '../../media/img/moorainbow/',
                selectText: charts_uiDictionary.selectColorAssign,
                onComplete: assignColors
            });
    }
}

function fixMinMaxDigitals() {
    for (var panelIndex = 1; panelIndex < _chart.panels.length; panelIndex++) {
        var panel = _chart.panels[panelIndex];
        if (panel.valueAxes[0].max == 10)
            panel.valueAxes[0].maximum = 1.2;
    }
}

function chartZoomed(event) {
    if (_markedLegendDivId != null) {
        renderMarker();
        redrawCharts();
    }
    buildStatTable(event);
}

function chartRendered() {
    bindChartEvents();
    manageBalloons();
    setHeights();
}

function chartInit() {
    if (typeof (window.hideWaitModal !== "undefined"))
        window.hideWaitModal();

    fixMinMaxDigitals();
    setDivChartSize();
    if (_chart != null) {
        _chart.invalidateSize();
    }
}

function getGuideData(axisId) {
    for (var i = 0; i < _axisGuides.length; i++) {
        if (_axisGuides[i].axisId == axisId)
            return _axisGuides[i];
    }
    return null;
}

function clearBands(e) {
    stopEventPropagation(e);

    if (_currentAxis != null) {
        var currentGuide = getGuideData(_currentAxis.axis.id);

        if (currentGuide != null) {

            _axisGuides.splice(_axisGuides.indexOf(currentGuide));
            setActualAxis(e);
            hideAllMenus();
            redrawChart();
        }
    }
}

function assignBands(e) {
    stopEventPropagation(e);

    if (_currentAxis != null) {
        var toreturn = false;

        if (!isFloat(_bands_tb_min.get('value'))) {
            $("minBandsRequisite").setStyle("display", "inline");
            toreturn = true;
        } else {
            $("minBandsRequisite").setStyle("display", "none");
        }

        if (!isFloat(_bands_tb_max.get('value'))) {
            $("maxBandsRequisite").setStyle("display", "inline");
            toreturn = true;
        } else {
            $("maxBandsRequisite").setStyle("display", "none");
        }

        if (toreturn) {
            return false;
        }

        var currentGuide = getGuideData(_currentAxis.axis.id);

        if (currentGuide != null) {
            currentGuide.min = _bands_tb_min.get('value').toFloat();
            currentGuide.max = _bands_tb_max.get('value').toFloat();
            currentGuide.inner = _bands_sel.getSelected().get("value") == "true" ? true : false;// _bands_cb.get('value');
        } else {
            var g = {
                min: _bands_tb_min.get('value').toFloat(),
                max: _bands_tb_max.get('value').toFloat(),
                inner: _bands_sel.getSelected().get("value") == "true" ? true : false,
                axisId: _currentAxis.axis.id
            };

            _axisGuides.push(g);
        }
        setActualAxis(e);
        hideAllMenus();
        redrawChart();
    }

    return false;
}

function isInt(n) {
    return n != null && (!isNaN(parseInt(n)) && isFinite(n));
}

function isFloat(n) {
    n = n.split(',').join(".");
    return n != null && (!isNaN(parseFloat(n)) && isFinite(n));
}

function changeBands(e) {
    stopEventPropagation(e);

    if (_currentAxis != null) {

        _axis_bands_control.setStyle('display', 'block');
        var coords = _contextMenuAxis.getCoordinates();
        var contextSize = _axis_bands_control.getSize();

        var currentGuide = getGuideData(_currentAxis.axis.id);

        if (currentGuide == null) {
            _bands_tb_min.set("value", _currentAxis.axis.min);
            _bands_tb_max.set("value", _currentAxis.axis.max);
            _bands_sel.set("value", "true");
        } else {
            _bands_tb_min.set("value", currentGuide.min);
            _bands_tb_max.set("value", currentGuide.max);
            _bands_sel.set("value", currentGuide.inner ? "true" : "false");
        }

        _axis_bands_control.setStyles({
            'left': coords.left - contextSize.x - leftContextMenuMargin,
            'top': coords.top - toolBarHeight
        });

        $('mooRainbow').setStyle('display', 'none');
        $('axis-min-max-control').setStyle('display', 'none');
    }
    return false;
}

function addGuide(from, to, axis) {
    var guide = new AmCharts.Guide();
    guide.value = from;
    guide.toValue = to;
    guide.fillColor = "#CC0000";
    guide.fillAlpha = .2;
    axis.addGuide(guide);
}

function setActualAxis(e) {
    stopEventPropagation(e);

    if (_currentAxis != null) {

        _currentAxis.chart.chartCursor.valueLineAxis = _currentAxis.axis;

        for (var i = 0; i < _currentAxis.chart.valueAxes.length; i++) {
            _currentAxis.chart.valueAxes[i].guides = [];
        }

        var currentGuide = getGuideData(_currentAxis.axis.id);

        if (currentGuide != null) {

            if (currentGuide.inner) {
                addGuide(currentGuide.min, currentGuide.max, _currentAxis.axis);
            } else {
                addGuide(currentGuide.min, _currentAxis.axis.min, _currentAxis.axis);
                addGuide(currentGuide.max, _currentAxis.axis.max, _currentAxis.axis);
            }
        }
        _currentAxis.chart.validateNow();
    }

    hideAllMenus();

    return false;
}

function preventClick(e) {
    stopEventPropagation(e);
}

function preventPostOnEnter(e) {
    if (e.code == 13) {
        stopEventPropagation(e);
        return false;
    }
    return true;
}

function assignMinMaxOnEnter(e) {
    if (e.code == 13) {
        assignAxis(e);
        return false;
    }
    return true;
}

function assignBandsOnEnter(e) {
    if (e.code == 13) {
        assignBands(e);
        return false;
    }
    return true;
}

function redrawCharts() {
    for (var panelIndex = 0; panelIndex < _chart.panels.length; panelIndex++) {
        var panel = _chart.panels[panelIndex];
        panel.validateNow();
    }    
}

function redrawChart() {
    _chart.validateNow();    
}

function hideAllCategoryAxisGuides(redraw) {
    for (var panelIndex = 0; panelIndex < _chart.panels.length; panelIndex++) {
        var panel = _chart.panels[panelIndex];
        panel.categoryAxis.guides = new Array();
    }
    if (redraw)
        redrawChart();
}

function stopEventPropagation(event, eventAternative) {
    var e = event;

    if (e == null && eventAternative != null)
        e = eventAternative;

    if (e == null)
        return false;

    if (e.stopPropagation) {
        e.stopPropagation();
    } else {
        e.returnValue = false;
        e.cancelBubble = true;
    }

    return false;
}

var svgns = "http://www.w3.org/2000/svg";

function makeSvg() {
    var svg = document.createElementNS("http://www.w3.org/2000/svg", "svg");
    svg.setAttribute('style', 'width:16px; height:16px');
    return svg;
}

function makeCircle(color) {
    var svg = makeSvg();

    var path = document.createElementNS(svg.namespaceURI, 'circle');

    path.setAttribute('fill', color);
    path.setAttribute('fill-opacity', "1");
    path.setAttribute('stroke', color);
    path.setAttribute('stroke-opacity', "1");
    path.setAttribute('stroke-width', "1");
    path.setAttribute('transform', "translate(8 8)");
    path.setAttribute('cx', "0");
    path.setAttribute('cy', "0");
    path.setAttribute('r', "8");

    svg.appendChild(path);
    return svg;
}

function makeTriangle(color) {
    var svg = makeSvg();

    var path = document.createElementNS(svg.namespaceURI, 'path');
    path.setAttribute('fill', color);
    path.setAttribute('fill-opacity', "1");
    path.setAttribute('stroke', color);
    path.setAttribute('stroke-opacity', "1");
    path.setAttribute('stroke-width', "1");
    path.setAttribute('transform', "translate(8 8)");
    path.setAttribute('d', "M -8 -8 L 0 8 L 8 -8 Z");
    path.setAttribute('cs', "100,100");
    svg.appendChild(path);

    return svg;
}

function buildResxLegend(color, isDigital) {

    var svg;
    if (isDigital) { svg = makeTriangle(color); }
    else { svg = makeCircle(color); }

    var td = new Element('td', { 'class': '' });
    td.grab(svg);
    return td;
}

function buildTable(headers) {
    var table = new Element('table', { 'class': '' });
    var thead = new Element('thead', { 'class': '' });
    var trHead = new Element('tr', { 'class': '' });

    thead.grab(trHead);

    headers.each(function (head) {
        trHead.grab(new Element('th', {
            'class': '',
            'html': head
        }));
    });

    table.grab(thead);

    return table;
}

function addStatGuide(value, color, id, label) {
    var guide = new AmCharts.Guide();
    guide.label = label;
    guide.position = "right";
    guide.lineThickness = 3;
    guide.dashLength = 5;
    guide.lineColor = color;
    guide.value = value;
    guide.id = id;

    return guide;
}

function getStatGuide(ax, id) {
    for (var guideIndex = 0; guideIndex < ax.guides.length; guideIndex++) {
        var gui = ax.guides[guideIndex];
        if (gui.id === id) {
            return gui;
        }
    };
    return null;
}

function removeStatGuides() {

    var panel = _chart.panels[0];
    for (var valueAxisIndex = 0; valueAxisIndex < panel.valueAxes.length; valueAxisIndex++) {
        var valueAxe = panel.valueAxes[valueAxisIndex];

        Array.each(["guide-stat-avg", "guide-stat-min", "guide-stat-max"], function (id) {
            var g = getStatGuide(valueAxe, id);
            if (g != null)
                valueAxe.removeGuide(g);
        });
    }
}

function toggleStatOnChart(event) {

    var control = $(this);

    removeStatGuides();

    var hasSelected = control.hasClass('display-stat-button-selected');
    $$('.display-stat-button').removeClass('display-stat-button-selected');

    if (!hasSelected) {
        control.addClass('display-stat-button-selected');

        var avg = control.get('data-avg');
        var min = control.get('data-min');
        var max = control.get('data-max');
        var color = control.get('data-color');
        var axisId = control.get('data-axis-id');

        var ax = getAnalogAxis(axisId);

        ax.addGuide(addStatGuide(avg, color, 'guide-stat-avg', 'AVERAGE'));
        ax.addGuide(addStatGuide(min, color, 'guide-stat-min', 'MIN'));
        ax.addGuide(addStatGuide(max, color, 'guide-stat-max', 'MAX'));
    }

    _chart.validateNow();    
    return false;
}

function getAnalogAxis(id) {
    var panel = _chart.panels[0];
    for (var valueAxisIndex = 0; valueAxisIndex < panel.valueAxes.length; valueAxisIndex++) {
        var valueAxe = panel.valueAxes[valueAxisIndex];
        if (valueAxe.id === id)
            return valueAxe;
    }
    return null;
}

var _zoomStartDate;
var _zoomEndDate;

function buildStatTable(event) {
    if (!_isStatVisible)
        return;

    // ignore zoom if the flag is set
    //if (_chart.ignoreZoom) {
    //    _chart.ignoreZoom = false;
    //    return;
    //}

    removeStatGuides();

    var statAnalog = [];
    var statDigital = [];

    var dataset = _chart.mainDataSet;
    var category = dataset.categoryField;

    var statData = [];

    for (var panelIndex = 0; panelIndex < _chart.panels.length; panelIndex++) {
        var panel = _chart.panels[panelIndex];

        for (var graphIndex = 0; graphIndex < panel.stockGraphs.length; graphIndex++) {
            var graph = panel.stockGraphs[graphIndex];
            var field = graph.valueField;
            var isAnalog = graph.type == "line";

            statData[field] = {
                label: graph.legendTextReal,
                color: graph.lineColor,
                isAnalog: isAnalog,
                total: 0,
                min: Number.MAX_VALUE,
                max: -Number.MAX_VALUE,
                prevValue: 0,
                posCount: 0,
                switchCount: 0,
                count: 0,
                valueAxisId: graph.valueAxis.id
            };
        }
    }

    _zoomStartDate = event.startDate;
    _zoomEndDate = event.endDate;

    var startDate = Date.parse(_zoomStartDate);
    var endDate = Date.parse(_zoomEndDate);

    var startDateString = AmCharts.formatDate(startDate, "DD MMM YYYY (HH:NN)");
    var endDateString = AmCharts.formatDate(endDate, "DD MMM YYYY (HH:NN)");

    for (var x in dataset.dataProvider) {
        var date = Date.parse(dataset.dataProvider[x][category]);

        if (date >= startDate && date <= endDate) {

            for (var field in statData) {
                var stat = statData[field];
                var currentValue = dataset.dataProvider[x][field];
                if (isNaN(currentValue)) {

                } else {
                    stat.count++;
                    if (stat.isAnalog) {

                        stat.total += currentValue;

                        if (currentValue < stat.min)
                            stat.min = currentValue;
                        if (currentValue > stat.max)
                            stat.max = currentValue;
                    } else {
                        if (currentValue > 0) {
                            stat.posCount++;
                            if (stat.prevValue == 0 && stat.count > 1)
                                stat.switchCount++;
                        }
                    }
                    stat.prevValue = currentValue;
                }
            }
        }
    }

    for (var field in statData) {
        var stat = statData[field];

        if (stat.label && stat.label !== '') {
            if (stat.isAnalog) {
                statAnalog.push({
                    name: stat.label,
                    color: stat.color,
                    valueAxisId: stat.valueAxisId,
                    average: stat.count <= 0 ? 'NA' : (stat.total / stat.count).toFixed(2),
                    min: stat.count <= 0 ? 'NA' : stat.min,
                    max: stat.count <= 0 ? 'NA' : stat.max,

                });
            } else {
                statDigital.push({
                    name: stat.label,
                    color: stat.color,
                    valueAxisId: stat.valueAxisId,
                    switchCount: stat.count <= 0 ? 'NA' : stat.switchCount,
                    percentUp: stat.count <= 0 ? 'NA' : (stat.posCount / stat.count * 100).toFixed(2)
                });
            }
        }
    }

    //_chart.ignoreZoom = true;_chart.validateData();
    //_chart.ignoreZoom = true;_chart.zoom(event.startDate, event.endDate);

    var tableDi = $('stat-tab-di-table');
    var tableAn = $('stat-tab-an-table');

    tableDi.setStyle('display', 'none');
    tableAn.setStyle('display', 'none');

    var statContainer = $('stat-box');

    statContainer.setStyle('display', 'block');

    if (statAnalog.length > 0) {
        tableAn.setStyle('display', 'block');
        var tbody = $('stat-tab-an-tbody');

        var dRanageAn = $('dateRangeAnalog');
        dRanageAn.set('html', 'From ' + startDateString + ' To ' + endDateString);
        tbody.empty();
        var rowIndex = 0;
        statAnalog.each(function (row) {
            var rowClass = '';
            if (rowIndex % 2 == 0)
                rowClass = 'odd';

            var tr = new Element('tr', { 'class': rowClass });
            tr.grab(buildResxLegend(row.color, false));
            tr.grab(new Element('td', { 'html': row.name, 'class': 'left' }));
            tr.grab(new Element('td', { 'html': row.average }));
            tr.grab(new Element('td', { 'html': row.min }));
            tr.grab(new Element('td', { 'html': row.max }));

            var a = new Element('a', {
                'href': '#',
                'class': 'display-stat-button',
                'data-avg': row.average,
                'data-min': row.min,
                'data-max': row.max,
                'data-color': row.color,
                'data-axis-id': row.valueAxisId
            });

            a.addEvent('click', toggleStatOnChart);

            tr.grab(new Element('td', {}).grab(a));

            tbody.grab(tr);
            rowIndex++;
        });
    }

    if (statDigital.length > 0) {
        var dRangeDi = $('dateRangeDigital');
        tableDi.setStyle('display', 'block');

        var tbody = $('stat-tab-di-tbody');
        tbody.empty();

        var rowIndex = 0;
        dRangeDi.set('html', 'From ' + startDateString + ' To ' + endDateString);

        statDigital.each(function (row) {
            var rowClass = '';
            if (rowIndex % 2 == 0)
                rowClass = 'odd';

            var tr = new Element('tr', { 'class': rowClass });
            tr.grab(buildResxLegend(row.color, true));
            tr.grab(new Element('td', { 'html': row.name, 'class': 'left' }));
            tr.grab(new Element('td', { 'html': row.switchCount }));
            tr.grab(new Element('td', { 'html': row.percentUp + " %" }));

            tbody.grab(tr);
            rowIndex++;
        });
    }
}

function renderMarker() {
    hideAllCategoryAxisGuides(false);
    _markedLegendDivId = _currentLegend.target.divId;

    var dataPanel = searchPanel(_currentLegend.target.divId, _chart.panels);

    if (dataPanel != null) {

        var graph = dataPanel.graphs[0];
        var field = graph.valueField;
        for (var panelIndex = 0; panelIndex < _chart.panels.length; panelIndex++) {

            var panel = _chart.panels[panelIndex];
            var dataLength = dataPanel.chartData.length;

            for (var i = 0; i < dataLength; i++) {
                var data = dataPanel.chartData[i];
                var value = data.dataContext[field + "High"];
                if (value !== undefined) {
                    if (value > 0 && guide == null) {
                        var guide = new AmCharts.Guide();
                        guide.date = Date.parse(data.time);

                        if (panel == dataPanel)
                            guide.fillColor = "#00CC00";
                        else
                            guide.fillColor = "#CC0000";

                        guide.fillAlpha = .2;
                        guide.lineThickness = 0;
                        guide.dashLength = 0;
                        guide.lineAlpha = 0;

                        guide.expand = false;
                    }
                    if ((value <= 0 || i == (dataLength - 1)) && guide != null) {
                        guide.toDate = Date.parse(data.time);
                        panel.categoryAxis.addGuide(guide);
                        guide = null;
                    }
                }
            }
        }
    }
}

function showHideMarker(e, forceRedraw) {
    stopEventPropagation(e, event);

    if (forceRedraw == null)
        forceRedraw = false;

    if (_currentLegend != null) {

        if (_markedLegendDivId == _currentLegend.target.divId && !forceRedraw) {
            _markedLegendDivId = null;
            hideAllCategoryAxisGuides(true);
            hideAllMenus();
            return false;
        }

        renderMarker();
    }
    redrawCharts();
    hideAllMenus();

    return false;
}

function assignColors(color) {
    if (_currentAxis != null) {
        _currentAxis.axis.axisColor = color.hex;
        _currentAxis.axis.color = color.hex;
        _currentAxis.axis.gridColor = color.hex;
        _currentAxis.chart.validateNow();
    }

    if (_currentLegend != null) {
        _currentLegend.dataItem.lineColor = color.hex;
        _currentLegend.dataItem.negativeLineColor = color.hex;
        _currentLegend.dataItem.chart.validateNow();
        buildStatTable({ startDate: _zoomStartDate, endDate: _zoomEndDate });        
    }
    hideAllMenus();
}

function assignAxis(e) {
    stopEventPropagation(e);
    var toReturn = false;
    if (!isInt(_axis_tb_min.get('value'))) {
        $("minRequisite").setStyle("display", "inline");
        toReturn = true;
    } else {
        $("minRequisite").setStyle("display", "none");
    }

    if (!isInt(_axis_tb_max.get('value'))) {
        $("maxRequisite").setStyle("display", "inline");
        toReturn = true;
    } else {
        $("maxRequisite").setStyle("display", "none");
    }

    if (toReturn)
        return;

    if (_currentAxis != null) {
        _currentAxis.axis.maximum = _axis_tb_max.get('value').toFloat();
        _currentAxis.axis.minimum = _axis_tb_min.get('value').toFloat();
        redrawChart();
        hideAllMenus();
    }
}

function searchPanel(legendDivId, panels) {
    for (var panelIndex = 0; panelIndex < panels.length; panelIndex++) {
        var panel = panels[panelIndex];
        var legend = panel.legend;
        if (legend.divId == legendDivId)
            return panel;
    }
    return null;
}

function bindChartEvents() {
    for (var panelIndex = 0; panelIndex < _chart.panels.length; panelIndex++) {
        var panel = _chart.panels[panelIndex];
        var legend = panel.legend;

        if (panelIndex == 0) {
            // Only analog panel
            for (var valueAxisIndex = 0; valueAxisIndex < panel.valueAxes.length; valueAxisIndex++) {
                var valueAxe = panel.valueAxes[valueAxisIndex];
                valueAxe.addListener("clickItem", showAxisMenu);
            }
        }
        legend.addListener("clickLabel", showLegendMenu);
        legend.addListener("clickMarker", clickLegendMarker);
    }
}

function setHeights() {
    if (_removedPanels.length == 0) {
        // This is the default case: AmCharts is rendering all panels, no post-invalidation tweaking is necessary.
        return;
    }
    var totalHeight = 0;
    var digitalHeight = 0;
    for (var i = 0; i < _chart.panels.length; i++) {
        totalHeight += _chart.panels[i].div.clientHeight;
        if (i == 1) digitalHeight = _chart.panels[i].div.clientHeight;
    }

    var visibleDigitalPanels = _chart.panels.length - 1 - _removedPanels.length;
    var analogPart = totalHeight - visibleDigitalPanels * digitalHeight;

    _chart.panels[0].div.setStyle('height', analogPart);
    for (var i = 1; i < _chart.panels.length; i++) {
        var digitalPanel = _chart.panels[i];
        if (_removedPanels.indexOf(digitalPanel) > -1) {
            digitalPanel.div.setStyle('display', 'none');
        } else {
            digitalPanel.div.setStyle('display', 'block');
        }
    }
}

function togglePanel(e) {
    stopEventPropagation(e);

    if (_currentLegend != null) {
        var panel = searchPanel(_currentLegend.target.divId, _chart.panels);
        if (_removedPanels.indexOf(panel) > -1) {
            // Panel to ADD            
            panel.div.setStyle("display", "block");
            _removedPanels.erase(panel);
        }
        else {
            // Panel to REMOVE         
			panel.div.setStyle("display", "none");            
            _removedPanels.push(panel);
        }

        redrawChart();
        hideAllMenus();
    }
}

function windowResize(e) {
    hideAllMenus();
    setDivChartSize();
    if (_chart != null) {
        _chart.invalidateSize();        
    }

    return false;
}

function setDivChartSize(numberOfDigitalPanels) {
    var chartDiv = $('chartdiv');

    var windowSize = window.getSize();

    var height = windowSize.y - toolBarHeight - chartPeriodSelectorHeight;

    if (_chart != null)
        numberOfDigitalPanels = _chart.panels.length - 1;

    var minHeight = (numberOfDigitalPanels * digitalPanelMinHeight) + analogPanelMinHeight;

    if (minHeight > height)
        height = minHeight;

    chartDiv.setStyle('height', height);
    chartDiv.setStyle('width', '100%');
    return false;
}

function toggleStat(e) {
    stopEventPropagation(e);
    var statContainer = $('stat-box');
    if (_isStatVisible) {
        statContainer.setStyle('display', 'none');

    } else {
        statContainer.setStyle('display', 'block');
    }
    _isStatVisible = !_isStatVisible;
    return false;
}

function changeColorAxis(e) {
    stopEventPropagation(e);
    _colorPicker.manualSet(_currentAxis.axis.axisColor, 'hex');
    _colorPicker.show('axis-context-menu', true);

    $('axis-context-menu').setStyle('display', 'block');
    var coords = _contextMenuAxis.getCoordinates();
    var contextSize = $('mooRainbow').getSize();

    $('mooRainbow').setStyles({
        'left': coords.left - contextSize.x,
        'top': coords.top
    });

    $('axis-min-max-control').setStyle('display', 'none');
    $('axis-bands-control').setStyle('display', 'none');

    return false;
}

function changeColorLegend(e) {
    stopEventPropagation(e);
    _colorPicker.manualSet(_currentLegend.dataItem.lineColor, 'hex');
    _colorPicker.show('analog-context-menu');
    return false;
}

function hideLegendMenus() {
    if (typeof (_contextMenuLegend) === "undefined")
        return false;

    _contextMenuLegend.setStyle('display', 'none');
    return false;
}

function hideAxisMenus() {
    if (typeof (_contextMenuAxis) === "undefined")
        return false;

    _contextMenuAxis.setStyle('display', 'none');
    _axis_min_max_control.setStyle('display', 'none');
    _axis_bands_control.setStyle('display', 'none');
    return false;
}

function hideAllMenus() {
    hideLegendMenus();
    hideAxisMenus();

    if (typeof (_colorPicker) === "undefined")
        return false;
    _colorPicker.hide();
    return false;
}

function showChangeAxis(e) {
    stopEventPropagation(e);
    _axis_min_max_control.setStyle('display', 'block');

    var coords = _contextMenuAxis.getPosition();
    var contextSize = _axis_min_max_control.getSize();

    _axis_min_max_control.setStyles({
        'left': coords.x - contextSize.x - leftContextMenuMargin,
        'top': coords.y - toolBarHeight
    });
    $('mooRainbow').setStyle('display', 'none');
    $('axis-bands-control').setStyle('display', 'none');

    return false;
}

function showAxisMenu(value) {
    stopEventPropagation(value.event, event);

    var actualMainMenu = $('actual-menuitem');

    hideAllMenus();
    _currentLegend = null;
    _currentAxis = value;
    _contextMenuAxis.setStyle('display', 'block');

    var contextSize = _contextMenuAxis.getSize();
    var scroll = window.getScroll();

    _contextMenuAxis.setStyles({
        'left': event.clientX - contextSize.x,
        'top': event.clientY + scroll.y - toolBarHeight
    });

    if (_chart.panels[0].valueAxes.length <= 1)
        actualMainMenu.setStyle('display', 'none');
    else
        actualMainMenu.setStyle('display', 'block');

    _axis_tb_min.set("value", _currentAxis.axis.min);
    _axis_tb_max.set("value", _currentAxis.axis.max);
    _axis_sp_step.set("text", _currentAxis.axis.step);

    return false;
}

function showLegendMenu(dataItem) {

    stopEventPropagation(dataItem.event, event);
    _currentLegend = dataItem;
    _currentAxis = null;
    var markerMenuItem = $('hide-show-marker-menuitem');
    var markerButton = $('hide-show-marker-button');

    markerMenuItem.setStyle('display', 'none');

    // WARNGING: Discover the legend type (based on Marker Type)
    if (_currentLegend.target.markerType == "triangleDown") {
        // DIGITAL
        // Check if panel is removed
        var panel = searchPanel(_currentLegend.target.divId, _removedPanels);
        if (panel == null) {
            // Panel is visible
            markerMenuItem.setStyle('display', 'block');
            if (_markedLegendDivId == _currentLegend.target.divId) {
                markerButton.set('html', charts_uiDictionary.legendUnmarker);
            } else {
                markerButton.set('html', charts_uiDictionary.legendMarker);
            }
        }
    } else {
        // ANALOG
    }

    var scroll = window.getScroll();

    _contextMenuLegend.setStyle('display', 'block');
    _contextMenuLegend.setStyles({
        'left': event.clientX,
        'top': event.clientY + scroll.y - toolBarHeight
    });

    $('analog-context-title').set('html', _currentLegend.dataItem.legendTextReal);

    return false;
}

function clickLegendMarker(dataItem) {
    _currentLegend = dataItem;
    if (_currentLegend.target.markerType == "triangleDown") {
        togglePanel(dataItem.event);
    }

    return false;
}

function manageBalloons() {
    Array.each(_chart.panels, function (panel) {
        panel.chartCursor.oneBalloonOnly = true;
        Array.each(panel.graphs, function (graph, j) {
            graph.showBalloon = !_isLegendVisible;
        });
    });
}

function printChartPortrait(e, settingDisplayLegend, settingDisplayStat, settingDisplayHeader) {
    printChart(null, true, settingDisplayLegend, settingDisplayStat, settingDisplayHeader);
    return false;
}

function printChartLandscape(e, settingDisplayLegend, settingDisplayStat, settingDisplayHeader) {
    printChart(null, false, settingDisplayLegend, settingDisplayStat, settingDisplayHeader);
    return false;
}

function printChart(e, portrait, settingDisplayLegend, settingDisplayStat, settingDisplayHeader) {

    stopEventPropagation(e);

    _chart.validateNow();    

    var height = 1123;
    var width = 794;

    if (!portrait) {
        height = 744;
        width = 1123;
    }

    var chDiv = $('data-container');
    var statDiv = $('stat-box');
    var printLegendContainer = $('legendContainerOnPrint');
    var headerDiv = $('chart-print-header');

    if (settingDisplayHeader)
        headerDiv.setStyle('display', 'block');

    var displayLegend = settingDisplayLegend;

    if (settingDisplayStat) {
        statDiv.setStyle('display', 'block');
        displayLegend = false;
    } else {
        statDiv.setStyle('display', 'none');
    }
    _chart.chartScrollbarSettings.enabled = false;

    var chartDivSize = chDiv.getSize();

    var legendHeight = 0;

    if (displayLegend) {

        printLegendContainer.setStyle('display', 'block');
        printLegendContainer.setStyle('width', width + "px");
        printLegendContainer.empty();

        _chart.panels.each(function(panel) {

            panel.graphs.each(function(graph) {
                var printLegendDiv = new Element('div',
                    {
                        'class': 'legend-print-box'
                    });

                var printLegendMarker = new Element('div',
                    {
                        'class': 'legend-print-marker'
                    });

                printLegendMarker.setStyle('border-left', 'solid 16px ' + graph.lineColor);

                var printLegendText = new Element('div',
                    {
                        'class': 'legend-print-text'
                    });

                printLegendText.set('html', graph.legendTextReal);

                var printLegendClear = new Element('div',
                    {
                        'class': 'legend-print-clear'
                    });

                printLegendDiv.grab(printLegendMarker);
                printLegendDiv.grab(printLegendText);
                printLegendDiv.grab(printLegendClear);

                printLegendContainer.grab(printLegendDiv);
            });
        });

        legendHeight = printLegendContainer.getSize().y;

        var temp = printLegendContainer.getDimensions();
    }

    chDiv.setStyle('width', width + "px");
    chDiv.setStyle('height', (height - legendHeight) + "px");

    redrawChart();
    window.print();

    _chart.chartScrollbarSettings.enabled = true;

    if (displayLegend) {
        printLegendContainer.setStyle('display', 'none');
    }

    headerDiv.setStyle('display', 'none');
    statDiv.setStyle('display', 'block');

    chDiv.setStyle('width', chartDivSize.x + "px");
    chDiv.setStyle('height', chartDivSize.y + "px");

    redrawChart();

    return false;
}