var dateTimePickerFrom;
var dateTimePickerTo;
var newProfileId = "new";

Element.implement({
    show: function () { return this.setStyle('display', ''); },
    hide: function () { return this.setStyle('display', 'none'); }
});

function setInitialPanelVisibility() {
    managePanelVisibilityByName("profiles", "show");
    managePanelVisibilityByName("time", "show");
    managePanelVisibilityByName("resources", "show");  
}

function showLegendOrListOfResources() {
    if (isChart() && $('resources-display-conf-tree').children.length > 0) {
        managePanelVisibilityByName("resources", "hide");
        managePanelVisibilityByName("legend", "show");
    } else {
        managePanelVisibilityByName("resources", "show");
        managePanelVisibilityByName("legend", "hide");
    }
    
}

function managePanelVisibilityByName(panelName, action) {
    managePanelVisibility(panelName + "-panel", panelName + "-button", action);
}

function managePanelVisibility(targetId, buttonId, action) {
    var targetPanel = $(targetId);
    var targetButton = $(buttonId);

    var isLegend = (targetId === "legend-panel");
    var lc = $('legendContainer');

    if (action === "toggle") {
        action = targetPanel.getStyle('display') === 'none' ? "show" : "hide";
    }

    if (action === "show") {
        if (isLegend) {
            lc.setStyle('left', '0px').setStyle('top', '0px'); /* HACK to force LEGEND SVG REDRAW */
        }
        targetButton.addClass('left-toolbar-button-active');
        targetPanel.show();
    } else {
        targetButton.removeClass('left-toolbar-button-active');
        targetPanel.hide();
    }

    setDataContainerSizes();
}

function managePanelVisibilityClick(target) {
    var _this = $(target);
    var targetId = _this.get('data-panel-target');
    var buttonId = _this.get('data-button-target');

    managePanelVisibility(targetId, buttonId, "toggle");
}

function isChart() {
    return $('hiddenRepresentationType').get("value") !== "table";
}

function displayWaitModal(showCancelButton, message) {
    $('modalPopUpContainerWait').style.display = 'block';
    if (showCancelButton)
        $('cancelLink').show();
    else
        $('cancelLink').hide();
}

function hideWaitModal() {

    $('labelCancelingMessage').hide();
    $('labelLoadingMessage').show();
    $('cancelLink').hide();

    $('modalPopUpContainerWait').style.display = 'none';
}

function refreshData() {

    displayWaitModal(true);

    if (!isChart()) {
        $('table-container').show();
        $('chart-container').hide();

        $('table-container').getChildren().each(function (f) {
            if (f.id != "toolHistoricalFiller0") {
                f.empty();
            }
        });

        window.startInvocation();
    }
    else {      
        $('table-container').hide();
        $('chart-container').show();
        window.updateChartData();
    }
}

function toolbarButtonClick(e) {
    stopEventPropagation(e);
    managePanelVisibilityClick(this);
    return false;
}

function refreshButtonClick(e) {
    stopEventPropagation(e);
    refreshData(this);
    return false;
}

function addOptionsToSelect(selectId, options, clearBeforeInsert) {
    var elSel = $(selectId);
    if (clearBeforeInsert)
        elSel.options.length = 0;
    for (var index = 0; index < options.length; index++) {
        var elOptNew = document.createElement('option');
        elOptNew.text = options[index].display;
        elOptNew.value = options[index].id;        
        try {
            elSel.add(elOptNew, null);
        } catch (ex) {
            elSel.add(elOptNew);
        }
    }
}

function updateTimeIntervalVisibilities() {
    var intervalTypesId = $('dropDownListIntervalTypes').get("value");
    if (intervalTypesId === "None") {
        $('time-interval-controls').show();
    } else {
        $('time-interval-controls').hide();
    }
}

function updateTimeIntervalData(timeIntervalData) {
    $('textBoxToDate').set('value', timeIntervalData.to);
    $('textBoxFromDate').set('value', timeIntervalData.from);

    dateTimePickerTo.refreshValue(timeIntervalData.to);
    dateTimePickerFrom.refreshValue(timeIntervalData.from);

    updateTimeIntervalSpan(timeIntervalData.span);
    $('dropDownListIntervalTypes').set("value", timeIntervalData.intervalTypesId);
}

function profileLoadButtonClick(e) {
    stopEventPropagation(e);

    var profileId = $('dropDownListProfiles').get("value");
    if (profileId === newProfileId)
        return;

    new Request({
        url: '/HistoricalInvocation.rix',
        onComplete: function (response) {
            var profileData = JSON.decode(response);

            updateTimeIntervalData(profileData);
            updateTimeIntervalVisibilities();

            updateCurrentResources();
            refreshData();
        },
        timeout: 500000
    }).send("modality=ajax&exec=loadprofile&profileId=" + profileId);

    return false;
}

function profileDeleteButtonClick(e) {
    stopEventPropagation(e);
    var profileId = $('dropDownListProfiles').get("value");
    if (profileId === newProfileId)
        return;

    var deleteConfirmText = $('hiddenDeleteConfirm').get("value");

    if (confirm(deleteConfirmText)) {
        new Request({
            url: '/HistoricalInvocation.rix',
            onComplete: function (response) {
                if (response === "OK") {
                    updateProfiles();
                }
            },
            timeout: 500000
        }).send("modality=ajax&exec=deleteprofile&profileId=" + profileId);
    }
    return false;
}

function profileSaveAsButtonClick(e) {
    stopEventPropagation(e);

    var profileName = encodeURI($('profile-input-name').get("value"));
    var profileIsHaccp = $('resource-selector-haccp').checked ? "true" : "false";
    var profileId = $('dropDownListProfiles').get("value");

    new Request({
        url: '/HistoricalInvocation.rix',
        onComplete: function(response) {
            updateProfiles(response);           
            $('profile-saveas-container').hide();
        },
        timeout: 500000
    }).send("modality=ajax&exec=saveprofile&profileName=" +
        profileName +
        "&profileId=" +
        profileId +
        "&profileIsHaccp=" +
        profileIsHaccp);

    return false;
}

function profileSaveButtonClick(e) {
    stopEventPropagation(e);

    $('profile-input-name').set("value", $('dropDownListProfiles').getSelected()[0].get('text'));
    $('profile-saveas-container').show();

    return false;
}

function dropDownListIntervalTypesChange(e) {
    stopEventPropagation(e);
    updateTimeIntervalVisibilities();
    timeStampChanged(e);
    return false;
}

function resourceSelectorCloseButtonClick() {
    $('modalPopUpContainerResources').hide();
    return false;
}

function updateSelectedResourcesHaccp(cabBeHaccp, isHaccp) {
    if (cabBeHaccp) {
        $('labelResourceSelectorAsHaccp').removeClass('disabled');
        $('resource-selector-haccp').disabled = false;
        $('resource-selector-haccp').checked = isHaccp;
        $('labelResourceSelectorHaccpHint').hide();
    } else {
        $('labelResourceSelectorAsHaccp').addClass('disabled');
        $('labelResourceSelectorHaccpHint').show();
        $('resource-selector-haccp').checked = false;
        $('resource-selector-haccp').set('disabled', 'disabled');
    }
}

function isSelectedResourcesHaccp() {
    var selectedResx = extractSelectedResources();
    new Request({
        url: '/HistoricalInvocation.rix',
        onComplete: function (response) {
            if (response === "YES") updateSelectedResourcesHaccp(true, false);
            else updateSelectedResourcesHaccp(false, false);
        },
        timeout: 500000
    }).send("modality=ajax&exec=iscurrentresourceshaccp&resources=" + selectedResx);
}

function extractSelectedResources() {
    var selectable = $$('input.sgnSelector');
    var selectedResx = "";
    selectable.each(function (currentCheckBox) {
        if (currentCheckBox.checked)
            selectedResx += currentCheckBox.name + ";";
    });

    return selectedResx;
}

function resourceSelectorClearFilters() {
    $('resource-selector-filter-device-input').set("value", "");
    $('resource-selector-filter-resource-input').set("value", "");
    $('resource-selector-filter-onlyslected').checked = false;
    resourceSelectorApplyFilters();
}

function resourceSelectorApplyFilters() {
    var toSearchDevices = $('resource-selector-filter-device-input').get("value").toLowerCase();
    var toSearchResources = $('resource-selector-filter-resource-input').get("value").toLowerCase();
    var toSearchOnlySelected = $('resource-selector-filter-onlyslected').checked == true;

    $$('table.sgnDevicesTable').each(function (deviceTable) {
        var isDeviceVisible = true;
        var deviceName = deviceTable.getElements('td.grdInstrumentName')[0].get('html').toLowerCase();
        if (toSearchDevices !== "" && (deviceName.indexOf(toSearchDevices) === -1))
            isDeviceVisible = false;
        
        if (isDeviceVisible) {

            var resources = deviceTable.getElements('.hdwResource');
            var visibleResx = 0;
            resources.each(function (r) {
                var resxName = r.getChildren('td.liteGridRowName')[0].get('html').toLowerCase();
                var resxChecked = r.getElement('input.sgnSelector').checked;

                if ((toSearchResources !== "" && (resxName.indexOf(toSearchResources) === -1)) || (toSearchOnlySelected && !resxChecked))
                {
                    r.addClass('suppressed');
                }
                else {
                    r.removeClass('suppressed');
                    visibleResx++;
                }
                    
            });
            if (visibleResx <= 0)
                isDeviceVisible = false;
        }

        if (isDeviceVisible) {
            deviceTable.removeClass('suppressed');
        } else {
            deviceTable.addClass('suppressed');
        }
    });
}

function resourceSelectorConfirmButtonClick() {
    var selectedResx = extractSelectedResources();
    var haccp = $('resource-selector-haccp').checked ? "1" : "0";

    if (selectedResx === "") {
        $('selectAtLeastOneDevice').show();
        return false;
    }

    $('selectAtLeastOneDevice').hide();
    new Request({
        url: '/HistoricalInvocation.rix',
        onComplete: function (response) {            
            updateCurrentResources();
            refreshData();
        },
        timeout: 500000
    }).send("modality=ajax&exec=setresources&resources=" + selectedResx + "&haccp=" + haccp);

    $('modalPopUpContainerResources').hide();

    return false;
}


function selectResourcesModalResize() {
    var height = window.getSize().y - 40;
    var width = window.getSize().x - 40;
    $('modalPopUpInnerContainerResources').setStyle('height', height + "px").setStyle('width', width + "px");
    $('resources-table-container').setStyle('width', width - 288 + "px");
}

function updateCurrentResources() {
    new Request({
        url: '/HistoricalInvocation.rix',
        onComplete: function (response) {
            var resxInfoData = JSON.decode(response);

            $('resources-display-conf-count').set("html", resxInfoData.configurationCount);

            var treeContainer = $('resources-display-conf-tree');
            treeContainer.empty();

            resxInfoData.interfaces.each(function (interface) {
                var interfaceDiv = new Element('div', {
                    'class': 'resources-display-interface'
                });
                interfaceDiv.set('html', interface.name);
                treeContainer.grab(interfaceDiv);
                interface.devices.each(function (device) {
                    var deviceDiv = new Element('div', {
                        'class': 'resources-display-device'
                    });
                    deviceDiv.set('html', device.name);
                    treeContainer.grab(deviceDiv);
                    device.resources.each(function (resource) {
                        var resourceDiv = new Element('div', {
                            'class': 'resources-display-resource'
                        });
                        resourceDiv.set('html', resource.name);
                        treeContainer.grab(resourceDiv);
                    });
                });
            });

            if (resxInfoData.canBeHaccp) {
                $('resources-display-conf-haccp').hide();
            } else {
                $('resources-display-conf-haccp').show();
            }

            $('resourcesThresholdMessage').hide();
            if (isChart() && resxInfoData.showResourcesThresholdWarning) {
                $('resourcesThresholdMessage').show();
            }

            showLegendOrListOfResources();
        },
        timeout: 500000
    }).send("modality=ajax&exec=getcurrentresources");
}

function selectResourcesButtonClick(e) {
    stopEventPropagation(e);
    displayWaitModal(false);
    new Request({
        url: '/HistoricalInvocation.rix',
        onComplete: function (response) {
            if (document.canceled) return;

            var resxData = JSON.decode(response);
            updateSelectedResourcesHaccp(resxData.canBeHaccp, resxData.isHaccp);
            $('resources-table-container').set("html", resxData.html);
            $('resource-selector-filter-onlyslected').checked = true;
            selectResourcesModalResize();
            // C&P from liteGrid.js (with changes)
            var deviceHeaders = document.getElements('.hdwDevice');
            deviceHeaders.forEach(function (deviceHeader) {
                var deviceCheckbox = deviceHeader.getElement('input');
                var resourceCheckboxes = deviceHeader.getNext().getElements('input');
                deviceCheckbox.addEvent('click', checkOnDeviceRow.pass({ device: deviceCheckbox, resources: resourceCheckboxes }, deviceCheckbox));
                deviceCheckbox.addEvent('click', function () {
                    isSelectedResourcesHaccp();
                });
                resourceCheckboxes.forEach(function (resourceCheckbox) {
                    resourceCheckbox.addEvent('click', checkOnResourceRow.pass({ device: deviceCheckbox, resources: resourceCheckboxes }, resourceCheckbox));
                    resourceCheckbox.addEvent('click', function () {
                        isSelectedResourcesHaccp();
                    });
                });
            });
            // C&P from televis.js
            var myTips = new Tips($$('.tipped'), { text: "tip", title: "tipTitle", showDelay: 500 });
            myTips.removeEvents('show').addEvent('show', function (tip, el) { tip.fade('in'); });
            myTips.removeEvents('hide').addEvent('hide', function (tip, el) { tip.fade('hide'); });

            $('resource-selector-filter-onlyslected').checked = extractSelectedResources() !== "";
            resourceSelectorApplyFilters();

            if ($('resource-selector-filter-onlyslected').checked == true) {
                window.liteGridExpandAll();
            }

            hideWaitModal();
          
            $('modalPopUpContainerResources').style.display = 'block';
        },
        timeout: 500000
    }).send("modality=ajax&exec=getselectableresources");

    return false;
}

Number.prototype.pad = function (size) {
    var s = String(this);
    while (s.length < (size || 2)) { s = "0" + s; }
    return s;
}

function updateTimeIntervalSpan(value) {
    $('labelTimeIntervalSpan').set('html', "(" + value + ")");
}

function timeStampChanged(e) {
    stopEventPropagation(e);

    var f = dateTimePickerFrom.choice;
    var t = dateTimePickerTo.choice;
    var type = $('dropDownListIntervalTypes').get("value");

    var from = "";
    var to = "";

    if (typeof (f.year) !== "undefined")
        from = f.year.pad(4) + (f.month + 1).pad(2) + f.day.pad(2) + f.hours.pad(2) + f.minutes.pad(2);

    if (typeof (t.year) !== "undefined")
        to = t.year.pad(4) + (t.month + 1).pad(2) + t.day.pad(2) + t.hours.pad(2) + t.minutes.pad(2);

    new Request({
        url: '/HistoricalInvocation.rix',
        onComplete: function (response) {
            var timeIntervalData = JSON.decode(response);
            updateTimeIntervalData(timeIntervalData);
            updateTimeIntervalVisibilities();
            //updateTimeIntervalSpan(response);
        },
        timeout: 500000
    }).send("modality=ajax&exec=settimestamp&type=" + type + "&from=" + from + "&to=" + to);

    return false;
}

function isToolbarPanelsVisible() {

    var visible = false;

    $$('.left-toolbar-panel').each(function (toolbarPanel) {
        if (toolbarPanel.getStyle('display') !== "none")
            visible = true;
    });
    return visible;
}

function setDataContainerSizes() {
    var height = window.getSize().y - 134;
    var width = window.getSize().x - 50;

    if (isToolbarPanelsVisible()) {
        width = width - 320;
    }

    $('data-container').setStyle('width', width + "px").setStyle('min-height', height + "px");

    if (isChart() && typeof (window.windowResize) != "undefined")
        window.windowResize();
}


function onWindowResize() {
    setDataContainerSizes();
    selectResourcesModalResize();
}

function updateProfiles(selectedValue) {
    new Request({
        url: '/HistoricalInvocation.rix',

        onComplete: function (response) {
            addOptionsToSelect("dropDownListProfiles", JSON.decode(response), true);
            updateProfilesVisibilities();
            if (selectedValue) {
                $('dropDownListProfiles').set('value', selectedValue);
            }
        },

        timeout: 500000
    }).send("modality=ajax&exec=getprofiles");

    return false;
}

function printButtonClick(e) {
    stopEventPropagation(e);
    if (isChart()) {
        var showHeader = $('print-include-header').checked;
        var showStat = $('print-include-statistics').checked;
        var showLegend = $('print-include-legend').checked;

        var today = new Date();
        var date = dateTimePickerFrom.format(today, " - d-m-Y @ H:i");
        $('print-date').set('html', date);

        if ($('print-layout-landscape').checked) {
            window.printChartLandscape(e, showLegend, showStat, showHeader);
        } else {
            window.printChartPortrait(e, showLegend, showStat, showHeader);
        }
    } else {
        window.location = '/Data/Historical.aspx?action=PRINT_TABLE';
    }
    return false;
}

function exportButtonClick(e) {
    stopEventPropagation(e);
    window.location = '/Data/Historical.aspx?action=EXPORT_DATA';
    return false;
}

function updateProfilesVisibilities() {
    $('profile-saveas-container').hide();

    var profileId = $('dropDownListProfiles').get('value');

    if (profileId === newProfileId) {
        $('profile-load-button').hide();
        $('profile-delete-button').hide();
    }
    else {
        $('profile-load-button').show();
        $('profile-delete-button').show();
    }
}

function radioButtonArchiveUseChanged() {

    var label = "";
    var sampAprox = $('tbArchiveHistoricalAprxSamples').get("value");
    var sampMult = $('tbArchiveEnergyDifferentialMultiplier').get("value");

    if ($('radioButtonArchiveUseHistoricalData').checked)
        label = "None";

    if ($('radioButtonArchiveUseHistoricalDataWithUndersampling').checked)
        label = "NoneUndersampling";

    if ($('radioButtonArchiveUseTemporary').checked)
        label = "Quick";

    if ($('radioButtonArchiveUseEnergy').checked)
        label = "Energy";

    new Request({
        url: '/HistoricalInvocation.rix', onComplete: function (response) {
            //refreshData();
        }, timeout: 500000
    }).send("modality=ajax&exec=setarchive&label=" + label + "&sampAprox=" + sampAprox + "&sampMult=" + sampMult);

    updateArchiveVisibilities();
}

function updateArchiveVisibilities() {
    $('boxArchiveEnergyDifferentialMultiplier').hide();
    $('boxArchiveHistoricalAprxSamples').hide();

    if ($('radioButtonArchiveUseHistoricalDataWithUndersampling').checked) {
        $('boxArchiveHistoricalAprxSamples').show();
    }
    if ($('radioButtonArchiveUseEnergy').checked) {
        $('boxArchiveEnergyDifferentialMultiplier').show();
    }
}

function timeIntervalMoveNextButtonClick(e) {
    stopEventPropagation(e);
    new Request({
        url: '/HistoricalInvocation.rix', onComplete: function (response) {
            var profileData = JSON.decode(response);
            updateTimeIntervalData(profileData);
            updateTimeIntervalVisibilities();
            refreshData();
        }, timeout: 500000
    }).send("modality=ajax&exec=timeintervalmovenext");
    return false;
}

function timeIntervalMovePrevButtonClick(e) {
    stopEventPropagation(e);
    new Request({
        url: '/HistoricalInvocation.rix', onComplete: function (response) {
            var profileData = JSON.decode(response);
            updateTimeIntervalData(profileData);
            updateTimeIntervalVisibilities();
            refreshData();
        }, timeout: 500000
    }).send("modality=ajax&exec=timeintervalmoveprev");
    return false;
}

function dropDownListProfilesChange(e) {
    stopEventPropagation(e);
    updateProfilesVisibilities();
    return false;
}

function cancelDataRequestInvocation() {
    document.canceled = true;
    $('cancelLink').setStyle('display', 'none');
    $('labelLoadingMessage').setStyle('display', 'none');
    $('labelCancelingMessage').setStyle('display', 'inline');

    if (isChart()) {
        new Request({
            url: '/HistoricalChartInvocation.rix',
            onComplete: function (response) {
                hideWaitModal();
            }
        }).send("modality=ajax&exec=cancel");
    } else {
        window.cancelInvocation();
    }
}


window.addEvent('domready', function () {

    if (!isChart()) {
        $('legend-button').hide();
        $('print-chart-settings').hide();
    }

    setDataContainerSizes();

    $('cancelLink').addEvent('click', cancelDataRequestInvocation);

    $$('.panel-activator').addEvent('click', toolbarButtonClick);
    $$('.panel-close-button').addEvent('click', toolbarButtonClick);

    var aproxVal = $('tbArchiveHistoricalAprxSamples').get("value");
    //BEGIN ARCHIVE
    var aprox = new QTYBox('tbArchiveHistoricalAprxSamples', {
        maxValue: 1000,
        onChange: function (qtybox) {
            radioButtonArchiveUseChanged();
        }
    }).setValue(aproxVal);

    var multValue = $('tbArchiveEnergyDifferentialMultiplier').get("value");
    var mult = new QTYBox('tbArchiveEnergyDifferentialMultiplier', {
        maxValue: 1000,
        incValue: $('tbArchiveEnergyDifferentialMultiplier').get("value"),
        onChange: function (qtybox) {
            radioButtonArchiveUseChanged();
        }
    }).setValue(multValue);

    updateArchiveVisibilities();

    $('radioButtonArchiveUseHistoricalData').addEvent('change', radioButtonArchiveUseChanged);
    $('radioButtonArchiveUseHistoricalDataWithUndersampling').addEvent('change', radioButtonArchiveUseChanged);
    $('radioButtonArchiveUseTemporary').addEvent('change', radioButtonArchiveUseChanged);
    $('radioButtonArchiveUseEnergy').addEvent('change', radioButtonArchiveUseChanged);

    //END ARCHIVE

    //BEGIN RESOURCES
    $('select-resources-button').addEvent('click', selectResourcesButtonClick);

    $('resource-selector-close-button').addEvent('click', resourceSelectorCloseButtonClick);
    $('resource-selector-expand-all-button').addEvent('click', window.liteGridExpandAll);
    $('resource-selector-collapse-all-button').addEvent('click', window.liteGridCollapseAll);

    $('resource-selector-select-all-button').addEvent('click', function () {
        window.liteGridSelectAll();
        isSelectedResourcesHaccp();
    });
    $('resource-selector-unselect-all-button').addEvent('click', function () {
        window.liteGridUnselectAll();
        isSelectedResourcesHaccp();
    });

    $('resource-selector-confirm-button').addEvent('click', resourceSelectorConfirmButtonClick);

    $('resource-selector-filter-device-input').addEvent('keyup', resourceSelectorApplyFilters);
    $('resource-selector-filter-device-input').addEvent('change', resourceSelectorApplyFilters);
    $('resource-selector-filter-resource-input').addEvent('keyup', resourceSelectorApplyFilters);
    $('resource-selector-filter-resource-input').addEvent('change', resourceSelectorApplyFilters);
    $('resource-selector-filter-onlyslected').addEvent('click', resourceSelectorApplyFilters);

    $('resource-selector-filter-clear').addEvent('click', resourceSelectorClearFilters);
    updateCurrentResources();

    // END RESOURCES

    // BEGIN INTERVAL TIMES
    $('dropDownListIntervalTypes').addEvent('change', dropDownListIntervalTypesChange);
    $('time-interval-button-move-next').addEvent('click', timeIntervalMoveNextButtonClick);
    $('time-interval-button-move-prev').addEvent('click', timeIntervalMovePrevButtonClick);

    dateTimePickerFrom = new DatePicker('textBoxFromDate',
        {
            pickerClass: 'datepicker_vista',
            timePicker: true,
            inputOutputFormat: 'd-m-Y @ H:i',
            format: 'd-m-Y @ H:i',
            allowEmpty: true,
            days: dateTime_daysOfWeekDictionary,
            months: dateTime_monthsDictionary,
            selectTimeMessage: dateTime_uiDictionary['selectTimeMessage'],
            confirmText: dateTime_uiDictionary['ok'],
            debug: false,
            onSelect: timeStampChanged
        });
    dateTimePickerTo = new DatePicker('textBoxToDate',
        {
            pickerClass: 'datepicker_vista',
            timePicker: true,
            inputOutputFormat: 'd-m-Y @ H:i',
            format: 'd-m-Y @ H:i',
            allowEmpty: true,
            days: dateTime_daysOfWeekDictionary,
            months: dateTime_monthsDictionary,
            selectTimeMessage: dateTime_uiDictionary['selectTimeMessage'],
            confirmText: dateTime_uiDictionary['ok'],
            debug: false,
            onSelect: timeStampChanged
        });

    dateTimePickerTo.refreshValue($('hiddenTimeIntervalTo').get("value"));
    dateTimePickerFrom.refreshValue($('hiddenTimeIntervalFrom').get("value"));

    updateTimeIntervalVisibilities();

    // END INTERVAL TIMES

    // BEGIN PROFILES

    //updateProfiles();
    //var profileId = $('hiddenProfileId').get("value");
    //$('dropDownListProfiles').set("value", profileId);
    updateProfilesVisibilities();

    $('dropDownListProfiles').addEvent('change', dropDownListProfilesChange);
    $('profile-load-button').addEvent('click', profileLoadButtonClick);
    $('profile-delete-button').addEvent('click', profileDeleteButtonClick);
    $('profile-save-button').addEvent('click', profileSaveButtonClick);
    $('profile-saveas-button').addEvent('click', profileSaveAsButtonClick);
    // END PROFILES


    // BEGIN PRINT
    $('print-button').addEvent('click', printButtonClick);
    $('export-button').addEvent('click', exportButtonClick);
    // END PRINT

    $('refresh-data-button').addEvent('click', refreshButtonClick);

    window.addEvent('resize', onWindowResize);

    if ($('hiddenRunAtStart').get("value") === "1")
        refreshData();

    setInitialPanelVisibility();
});
