﻿var recipientsValidation = recipientsValidation || {
};

recipientsValidation.Validator = function (validationType, allowMultiple) {  // recipientsValidation.MailValidation BEGIN

    var _validationType = validationType;
    var _allowMultiple = allowMultiple;
    // Private Fields
    var _recipientsTextBox = null;
    var _recipientsValidationStatus = null;
    var _recipientsValidationButton = null;
    var _renderedTooltipContent = null;

    var _canOpenPopup = false;
    var _testTooltipEnabled = false; // development only: when true display aside the button
    var _testTooltipContainer = null;

    var RECIPIENTS_STATUS_LOADING_CLASS = "recipients-status-loading";
    var RECIPIENTS_STATUS_ERROR_CLASS = "recipients-status-error";
    var RECIPIENTS_STATUS_VERIFIEDALL_CLASS = "recipients-status-verifiedAll";
    var RECIPIENTS_STATUS_VERIFIEDNONE_CLASS = "recipients-status-verifiedNone";
    var RECIPIENTS_STATUS_VERIFIEDSOME_CLASS = "recipients-status-verifiedSome";

    return {
        forceRefresh: forceRefresh,
        initialize: initialize
    };

    function initialize(recipientsTextBoxId, validateRecipientsButtonId) {
        _recipientsTextBox = $(recipientsTextBoxId);
        _recipientsValidationButton = $(validateRecipientsButtonId);

        _recipientsTextBox.addEvent('keyup', recipientsChangedDelayedFunction.pass(1000, this));
        _recipientsTextBox.addEvent('change', recipientsChangedDelayedFunction.pass(500, this));

        // The status element is handled client-side only
        _recipientsValidationStatus = new Element('div', {
            'class': 'recipients-status calendarTipped'
        });
        _recipientsTextBox.getParent().grab(_recipientsValidationButton);
        _recipientsTextBox.getParent().grab(_recipientsValidationStatus);

        _recipientsValidationButton.addEvent('click', function (event) {
            recipientsValidationButtonClicked();
        });

        // TOOLTIP SETUP
        // Tooltips

        var myTips = new Tips($$('.calendarTipped'), {
            text: "",
            title: "",
            showDelay: 500
        });

        myTips.removeEvents('show').addEvent('show', function (tip, el) {
            var tipel = this;
            if (_renderedTooltipContent) {
                tipel.setText(_renderedTooltipContent);
                tipel.toElement().setStyle('background-color', 'white');
                var renderedWidth = _renderedTooltipContent.getStyle('width');
                tipel.toElement().setStyle('width', 'calc(' + renderedWidth + ' + 10px)');
                tip.fade('in');
            }
            else {
                tip.fade('hide'); // avoid empty tooltip on first appearing
            }
        });

        myTips.removeEvents('hide').addEvent('hide', function (tip, el) {
            tip.fade('hide');
        });

    }

    function forceRefresh() {
        recipientsChanged();
    }

    function renderTooltip(jsonData) {

        var table = new Element('table', { 'class': 'validationTooltipTable' });
        table.set('cellpadding', '0');
        table.set('cellspacing', '0');
         

        var thead = new Element('thead', {});
        thead.inject(table);

        var trHeadEl = new Element('tr', {});
        trHeadEl.inject(thead);

        if (jsonData.Error) {
            // in case of error: a single cell.

            var thErr = new Element('th', {
                html: jsonData.Error
            });
            thErr.inject(trHeadEl);

            return table;
        }

        var th1 = new Element('th', {});
        th1.inject(trHeadEl);

        var th2 = new Element('th', {});
        th2.inject(trHeadEl);

        var th3 = new Element('th', {
            html: jsonData.HeaderValidatedAt
        });
        th3.inject(trHeadEl);

        var th4 = new Element('th', {
            html: jsonData.HeaderLastFailureAt
        });
        th4.inject(trHeadEl);


        var tbody = new Element('tbody', {});
        tbody.inject(table);

        for (var i = 0; i < jsonData.Validations.length; i++) {
            var trEl = new Element('tr', {});
            trEl.inject(tbody);

            var validationClass = getStatusClass(jsonData.Validations[i]);
            
            var td1 = new Element('td', {  });
            td1.inject(trEl);

            var validationStatus = new Element('div', { 'class': 'recipients-status ' + validationClass });
            validationStatus.inject(td1);
            
            var td2 = new Element('td', {
                html: jsonData.Validations[i].Value,
                'style': 'vertical-align:middle'
            });
            td2.inject(trEl); 
             
            var td3 = new Element('td', {
                html: jsonData.Validations[i].ValidatedAt,
                'style': 'vertical-align:middle;padding-left:10px'
            });
            td3.inject(trEl);

            var td4 = new Element('td', {
                html: jsonData.Validations[i].LastFailureAt,
                'style': 'vertical-align:middle;padding-left:10px'
            });
            td4.inject(trEl); 

        }

        return table;
    }

    // Used for single validation status and aggregated statuss
    function getStatusClass(jsonData) {
        if (jsonData) {
            if (jsonData.Error) {
                return RECIPIENTS_STATUS_ERROR_CLASS;
            }
            else if (jsonData.ValidatedThenFailed) {
                return RECIPIENTS_STATUS_VERIFIEDSOME_CLASS;
            }
            else if (jsonData.Validated) {
                return RECIPIENTS_STATUS_VERIFIEDALL_CLASS;
            }
            else if (jsonData.NeverValidated) {
                return RECIPIENTS_STATUS_VERIFIEDNONE_CLASS;
            }
        }
        return null;
    }

    function recipientsValidationButtonClicked() {
 
        var recipients = _recipientsTextBox.value;

        if (recipients && _canOpenPopup)
            networkExt.genericPopup.openPopup("RecipientsValidationPopup.aspx?recipentsForValidation=" + encodeURIComponent(recipients), 800, 500, false, true);
    }

    function recipientsChanged() {
        var newValue = _recipientsTextBox.value;
         
        setValidationStatus(RECIPIENTS_STATUS_LOADING_CLASS);

        var ajax = new Request({
            url: '/RecipientsValidationInvocation.rix',
            timeout: 10000,
            onTimeout: function () {
                console.log('RecipientsValidationInvocation.rix timeout');
                setValidationStatus(null, null);
            },
            onComplete: function (response) {
                var jsonData = JSON.decode(response);
                var renderedTooltipContent = renderTooltip(jsonData);
                var statusClass = getStatusClass(jsonData);
                setValidationStatus(statusClass, renderedTooltipContent);
            }
        });
 

        ajax.send("modality=ajax&query=" + _validationType + "&allowmultiple=" + _allowMultiple + "&recipients=" + encodeURIComponent(newValue));
    }

    function setValidationStatus(classToAdd, renderedTooltipContent) {
        _canOpenPopup = false;
        _renderedTooltipContent = '';
        _recipientsValidationStatus.removeClass(RECIPIENTS_STATUS_VERIFIEDALL_CLASS);
        _recipientsValidationStatus.removeClass(RECIPIENTS_STATUS_VERIFIEDNONE_CLASS);
        _recipientsValidationStatus.removeClass(RECIPIENTS_STATUS_VERIFIEDSOME_CLASS);
        _recipientsValidationStatus.removeClass(RECIPIENTS_STATUS_LOADING_CLASS);
        _recipientsValidationStatus.removeClass(RECIPIENTS_STATUS_ERROR_CLASS);

        if (classToAdd) {

            if (classToAdd !== RECIPIENTS_STATUS_LOADING_CLASS &&
                classToAdd !== RECIPIENTS_STATUS_ERROR_CLASS) {
                _canOpenPopup = true;
            }

            _recipientsValidationStatus.addClass(classToAdd);
            _renderedTooltipContent = renderedTooltipContent;

            if (_testTooltipEnabled && _renderedTooltipContent) {
                if (!_testTooltipContainer) {
                    _testTooltipContainer = new Element('div', { 'style' : 'float:right'});
                    _recipientsTextBox.getParent().grab(_testTooltipContainer, 'after');
                }

                _testTooltipContainer.empty();
                _renderedTooltipContent.inject(_testTooltipContainer);  
            }
        }
    } 
     
    /*DELAYED EVENT */
    function recipientsChangedDelayedFunction(p) {

        timedelayRecipientsStatusUpdate = p || 0; // timeout to handle multiple update sequences

        if ($chk(this.timedelayRecipientsStatusUpdate_ticket)) { $clear(this.timedelayRecipientsStatusUpdate_ticket); };
        this.timedelayRecipientsStatusUpdate_ticket = recipientsChanged.delay(timedelayRecipientsStatusUpdate, this);
    }

    function recipientsChangedDelayed() {
        recipientsChanged();
    }
    /*DELAYED EVENT */

}; // recipientsValidation.MailValidation END